/*
 * driver.h
 *-
 * Copyright (c) 2001, 2002, 2003 Corrado Santoro (csanto@diit.unict.it)
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Id: driver.h,v 1.1.1.1 2005/05/15 09:49:59 corrado Exp $
 */

#ifndef __DRIVER_H
#define __DRIVER_H

#include <kernel/ntypes.h>
#include <kernel/file.h>
#include <kernel/stat.h>

typedef int (*file_drv_open)(file *f,char * name,int mode);
typedef int32 (*file_drv_read)(file * f,char * buf,int32 len);
typedef int32 (*file_drv_write)(file * f,char * buf,int32 len);
typedef int (*file_drv_close)(file * f);
typedef int (*file_drv_ioctl)(file *f,uint32 cmd,void * args);
typedef int (*file_drv_fstat)(file *f, struct stat * sb);
typedef int32 (*file_drv_lseek)(file *f,int32 offset,int whence);

typedef struct
{
  file_drv_open  open;
  file_drv_read  read;
  file_drv_write write;
  file_drv_close close;
  file_drv_ioctl ioctl;
  file_drv_lseek lseek;
  file_drv_fstat fstat;
} t_file_drv_operations;

#define MAX_DRV_NAME  16

typedef struct file_drv_entry
{
  char  name[MAX_DRV_NAME+1];
  t_file_drv_operations  operations;
  struct file_drv_entry  *next;
} t_file_drv_entry;

int find_file_driver(char * name,t_file_drv_entry **drv);
int register_file_driver(char * name,t_file_drv_operations * ops);
int unregister_file_driver(char * name);
void driver_init(void);

#endif
