/*
 * file.h
 *-
 * Copyright (c) 2001, 2002, 2003 Corrado Santoro (csanto@diit.unict.it)
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Id: file.h,v 1.1.1.1 2005/05/15 09:49:59 corrado Exp $
 */

#ifndef __FILE_H
#define __FILE_H

#include <kernel/ntypes.h>
#include <kernel/fileconst.h>
#include <kernel/stat.h>

#define MAX_FILENAME  256
#define MAX_FILES     32

#define FILE_DRIVER(f)    ( (t_file_drv_entry *)f->driver)

#define BLOCK_SIZE    512

typedef struct {
  int   ref_count;
  char  name[MAX_FILENAME+1];
  int   mode;
  void  * driver;
  void  * private_data;
  char  eof;
  long  file_position;
  //long  file_length;
  char  current_block [BLOCK_SIZE];
  int   block_size;
  int   current_block_offset;
  int   isfs;
  struct stat stat_buffer;
} file;

#define FEOF(f)   (f->file_position >= f->file_length)

int sys_open(char * name,int mode);
long sys_write(int fid,char * buf,long size);
long sys_read(int fid,char * buf,long size);
long sys_ioctl(int fid,int cmd,void * arg);
int32 sys_lseek(int fid,int32 offset,int whence);
int sys_fstat(int fid, struct stat * sb);
int sys_close(int fid);

#endif
