/*
 * filesystems.h
 *-
 * Copyright (c) 2001, 2002, 2003 Corrado Santoro (csanto@diit.unict.it)
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Id: filesystems.h,v 1.1.1.1 2005/05/15 09:49:59 corrado Exp $
 */

#ifndef __FILESYSTEMS_H
#define __FILESYSTEMS_H

#include <kernel/ntypes.h>
#include <kernel/file.h>

typedef int (*file_sys_open)(file *f, char * name, int mode);
typedef int32 (*file_sys_read)(file * f, char * buf, int32 len);
typedef int32 (*file_sys_write)(file * f, char * buf, int32 len);
typedef int (*file_sys_close)(file * f);
typedef int (*file_sys_ioctl)(file *f, int cmd, void * args);
typedef int (*file_sys_lseek)(file *f, long offset, int whence);

typedef struct
{
  file_sys_open  open;
  file_sys_read  read;
  file_sys_write write;
  file_sys_close close;
  file_sys_ioctl ioctl;
  file_sys_lseek lseek;
} t_file_sys_operations;

#define MAX_FILESYS_NAME  16

typedef struct file_sys_entry
{
  char  name [MAX_FILESYS_NAME + 1];
  t_file_sys_operations  operations;
  struct file_sys_entry  *next;
} t_file_sys_entry;

int find_file_system (char * name, t_file_sys_entry **sys);
int register_file_system (char * name, t_file_sys_operations * ops);
int unregister_file_system (char * name);
void filesystem_init (void);

#endif
