/*
 * gdt.h
 *-
 * Copyright (c) 2001, 2002, 2003 Corrado Santoro (csanto@diit.unict.it)
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Id: gdt.h,v 1.1.1.1 2005/05/15 09:49:59 corrado Exp $
 */

#ifndef __GDT_H
#define __GDT_H

#include <kernel/ntypes.h>

#pragma pack(1)
typedef struct {
  uint16  limit_lo;
  uint16  base_lo;
  uint16  type; // it contains also base 23..16, Type, DPL and P
  uint16  base_limit_hi;  // it contains base 31..24, limit 19..16 and G
} gdt_entry;

typedef struct {
  uint16  offset_lo;
  uint16  selector;
  uint16  type;
  uint16  offset_hi;
} gdt_gate_type;
#pragma pack()

/*
 * GDT Struct
 * 31                16                0
 * +---------------------------------+-----------------------+
 * |      Segment base 15..0         |  Segment Limit 15..0  |
 * +---------------------------------+-----------------------+
 * |Base 31..24|G|0|0|0|Limit 19..16 |P|DPL|0|Type|Base23..16|
 * +-------------------+-------------+-+---+-+----+----------+
 * DPL = 2bit
 * Type = 4bit
 */

void display_gdt(uint16 selector);
void gdt_make_tss(uint16 selector,
                uint32 segment_base,uint32 segment_limit,uint16 dpl);
void gdt_make_data_segment(uint16 selector,
                  uint32 segment_base,uint32 segment_limit,uint16 dpl,
                  uint8 granul,uint8 writ,uint8 dir);
void gdt_make_code_segment(uint16 selector,
                  uint32 segment_base,uint32 segment_limit,uint16 dpl,
                  uint8 granul,uint8 rd,uint8 conf);

#endif
