/*
 * hash.h
 *-
 * Copyright (c) 2001, 2002, 2003 Corrado Santoro (csanto@diit.unict.it)
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Id: hash.h,v 1.1.1.1 2005/05/15 09:49:59 corrado Exp $
 */

#ifndef __HASH_H
#define __HASH_H

#include <kernel/wait.h>

typedef struct t_hash_element {
  void  * key;
  void  * data;
  struct t_hash_element * next;
} t_hash_element;

typedef struct t_hash_element * p_hash_element;

#define HASH_DEFAULT_SIZE 4

typedef struct {
  int size;
  int hash_type;
  p_hash_element bucket[HASH_DEFAULT_SIZE];
  t_semaphore mutex;
} t_hash_table;

#define HASH_TYPE_STRING  0
#define HASH_TYPE_INTEGER 1

t_hash_table * new_hash(int hash_type);
int add_hash(t_hash_table * table,void * key,void * data);
int not_atomic_add_hash (t_hash_table * table,void * key,void * data);
t_hash_element * find_hash_element(t_hash_table * table,void * key);
void * find_hash(t_hash_table * table,void * key);
t_hash_element * not_atomic_find_hash(t_hash_table * table,void * key);
int find_and_delete_hash(t_hash_table * table,void * key,t_hash_element * el);
void delete_hash(t_hash_table * table);
void critical_exit_hash(t_hash_table * table);
void critical_enter_hash(t_hash_table * table);

#endif
