/*
 * nufs.h
 *-
 * Copyright (c) 2001, 2002, 2003 Corrado Santoro (csanto@diit.unict.it)
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Id: nufs.h,v 1.1.1.1 2005/05/15 09:49:59 corrado Exp $
 */

/* NUXI File System data structures */

#ifndef __NUFS_H
#define __NUFS_H

#include <kernel/ntypes.h>

#define NUFS_MAX_FILENAME_LENGTH 64
#define NUFS_MAX_BLOCKS          13
#define NUFS_DIRECT_BLOCKS       10

#define NUFS_TYPE_REGULAR         0
#define NUFS_TYPE_DIRECTORY       1

#define NUFS_MAGIC  0x4E555849

#pragma pack(1)
typedef struct
{
  uint8   type;
  uint16  bits;
  uint16  links;
  uint32  size;
  uint32  blocks [NUFS_MAX_BLOCKS];
} t_nufs_disk_inode;

typedef struct
{
  uint8   name [NUFS_MAX_FILENAME_LENGTH + 1];
  uint32  inode;
} t_nufs_disk_dir_entry;

typedef struct
{
  uint32  magic_number;
  uint16  block_size;
  uint32  max_inodes;
  uint32  max_blocks;
  uint32  free_inodes;
  uint32  free_blocks;
  uint16  inodes_per_block;
  uint16  inode_list_size;
  uint16  block_list_size;
  uint32  root_inode;
}
t_nufs_disk_superblock;

typedef struct
{
  t_nufs_disk_superblock  * sb;
  uint32  * inode_free_list;
  uint32  * block_free_list;
  int     fd;
  char    * buffer;
} t_nufs_mem_superblock;

#pragma pack()

#endif
