/*
 * queue.h
 *-
 * Copyright (c) 2001, 2002, 2003 Corrado Santoro (csanto@diit.unict.it)
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Id: queue.h,v 1.1.1.1 2005/05/15 09:49:59 corrado Exp $
 */

#ifndef __QUEUE_H
#define __QUEUE_H

#include <kernel/wait.h>

typedef struct {
  int size;
  int num;
  int head;
  int tail;
  uint8  * data;
  t_semaphore mutex;
  t_cond  full,empty;
} t_stream;

t_stream * stream_new(int size);
void stream_delete(t_stream * q);
void stream_putbyte(t_stream * q,char data);
char stream_getbyte(t_stream * q);
void stream_put(t_stream * q,char * buf,int size);
void stream_get(t_stream * q,char * buf,int size);

typedef struct {
  int   size;
  char  *data;
} t_message_block;

typedef struct {
  int size;
  int num;
  int head;
  int tail;
  t_message_block *data;
  t_semaphore mutex;
  t_cond  full,empty;
} t_block_queue;

t_block_queue * block_queue_new(int size);
void block_queue_delete(t_block_queue * q);
void block_queue_put(t_block_queue * q,char * buf,int size);
int block_queue_get(t_block_queue * q,char * buf,int maxsize);

#endif
