/*
 * rpc.h
 *-
 * Copyright (c) 2001, 2002, 2003 Corrado Santoro (csanto@diit.unict.it)
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Id: rpc.h,v 1.1.1.1 2005/05/15 09:49:59 corrado Exp $
 */

#ifndef __RPC_H
#define __RPC_H

#include <kernel/wait.h>

#define MAX_PROCEDURE_NAME  32
#define MAX_PROGRAM_NAME    32
#define MAX_PARAMS          32

typedef uint32 (*rpc_proc)(uint32 *);

typedef struct {
  char name[MAX_PROCEDURE_NAME];
  rpc_proc proc;
} t_rpc_proc_entry;

typedef struct {
  int svr_id;
  char name[MAX_PROGRAM_NAME];
  int procedure_count;
  int max_procedure;
  t_rpc_proc_entry *procs;
  uint32 task_id;
  t_semaphore mutex;
} t_rpc_server;

typedef struct t_rpc_svr_list {
  t_rpc_server server;
  struct t_rpc_svr_list * next;
} t_rpc_svr_list;


#define RPC_CONNECT     0
#define RPC_CALL        1
#define RPC_DISCONNECT  2

#define RPC_RESULT      0

typedef struct
{
  int   cmd;
  char  p_name[MAX_PROCEDURE_NAME];
  t_semaphore * sync_semaphore;
  int32 * params;
} t_rpc_request;

t_rpc_server * rpc_create_program(char * name,int maxproc);
int rpc_register_call(t_rpc_server * svr,char * procname,rpc_proc proc);
void rpc_start(t_rpc_server * svr);

int rpc_connect (char * name);
int rpc_call (int pgm, char * fname,uint32 * params);
int rpc_disconnect (int pgm);

#endif
