/*
 * sys/stat.h
 *-
 * Copyright (c) 2001, 2002, 2003 Corrado Santoro (csanto@diit.unict.it)
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Id: stat.h,v 1.1.1.1 2005/05/15 09:49:59 corrado Exp $
 */

#ifndef __STAT_H
#define __STAT_H

#include <kernel/ntypes.h>
#include <user/time.h>

typedef uint32  dev_t;
typedef uint32  ino_t;
typedef uint16  mode_t;
typedef uint16  nlink_t;
typedef uint32  off_t;
typedef uint32  uid_t;
typedef uint32  gid_t;

struct stat {
  dev_t     st_dev;          /* inode's device */
  ino_t     st_ino;          /* inode's number */
  mode_t    st_mode;         /* inode protection mode */
  nlink_t   st_nlink;        /* number of hard links */
  uid_t     st_uid;          /* user ID of the file's owner */
  gid_t     st_gid;          /* group ID of the file's group */
  dev_t     st_rdev;         /* device type */
  time_t    st_atime;        /* time of last access */
  long      st_atimensec;    /* nsec of last access */
  time_t    st_mtime;        /* time of last data modification */
  long      st_mtimensec;    /* nsec of last data modification */
  time_t    st_ctime;        /* time of last file status change */
  long      st_ctimensec;    /* nsec of last file status change */
  off_t     st_size;         /* file size, in bytes */
  uint64    st_blocks;       /* blocks allocated for file */
  uint32    st_blksize;      /* optimal blocksize for I/O */
  uint32    st_flags;        /* user defined flags for file */
  uint32    st_gen;          /* file generation number */
  uint32    st_lspare;
  uint64    st_qspare[2];
};

#endif

