/*
 * task.h
 *-
 * Copyright (c) 2001, 2002, 2003 Corrado Santoro (csanto@diit.unict.it)
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Id: task.h,v 1.1.1.1 2005/05/15 09:49:59 corrado Exp $
 */

#ifndef __TASK_H
#define __TASK_H

#include <kernel/ntypes.h>
#include <kernel/file.h>
#include <user/sys/time.h>

#define TASK_STARTUP    0
#define TASK_RUNNING    1
#define TASK_READY      2
#define TASK_SLEEPING   3
#define TASK_STOPPED    4
#define TASK_ENDING     5

#pragma pack(1)
typedef struct TSS32 {
   uint16 link, __unused0;
   uint32 esp0;
   uint16 ss0, __unused1;
   uint32 esp1;
   uint16 ss1, __unused2;
   uint32 esp2;
   uint16 ss2, __unused3;
   uint32 cr3, eip, eflags;
   uint32 eax,ecx,edx,ebx,esp,ebp,esi,edi;
   uint16 es, __unused4;
   uint16 cs, __unused5;
   uint16 ss, __unused6;
   uint16 ds, __unused7;
   uint16 fs, __unused8;
   uint16 gs, __unused9;
   uint16 ldt, __unused10;
   uint16 debugtrap, iomapbase;
} TSS;
#pragma pack(1)

#define TASK_STACK_SIZE   8192

typedef void (*startup_proc)(void *);

#define MAX_SYSCALL_PARAMS  32

typedef struct {
  uint32  syscall_no;
  uint32  params[MAX_SYSCALL_PARAMS];
  uint32  return_value;
} t_syscall_p;

struct sched_param {
  int sched_priority;
}; 

#define MAX_SIGNAL  32

typedef struct task_struct {
  TSS     tss;
  uint8   *stack;
  uint32  priority;
  uint32  tid;
  uint32  p_tid;
  uint8   state;
  uint8   saved_state;
  uint32  exit_code;
  file  * files[MAX_FILES];
  struct timeval  cpu_time;
  t_syscall_p  syscall;
  int     errno;
  //uint8   fork_request;
  //int     fork_ret_val;
  //struct task_struct * forked_task;
  uint32  signal_bit_vector;
  void    * signal_proc_vector[MAX_SIGNAL];
  int     signal_handled;
  TSS     saved_tss;
  struct task_struct * next;
  struct task_struct * next_ordered, * prev_ordered;
} task_struct;


/*
 priority assignment:
 0 - 100000 realtime (static) priorities
 200000 default priority
 */
#define DEFAULT_PRIORITY  (uint32)200000

typedef struct {
  task_struct * begin;
  task_struct * end;
  int total_tasks,last_tid;
} t_task_list;

extern task_struct * current_task;
extern t_task_list task_list;

void task_init(void);
void task_prepare(task_struct * task,startup_proc proc,void * param);
void task_add(uint32 *tid,startup_proc proc,void * param);
void task_fork_prepare(task_struct * parent,void * proc);
void task_end(uint32 end_code);
void task_change_priority (task_struct * task, uint32 newprio);
int  task_change_priority_by_tid (int tid, uint32 newprio);
void task_remove (task_struct * task);
void task_dump(void);
task_struct * task_find (int tid);
task_struct * task_find_by_parent (int tid);
void scheduler(void);

extern int scheduler_lock;

#define LOCK_SCHEDULER    { scheduler_lock++; }
#define UNLOCK_SCHEDULER  { scheduler_lock--; }

#endif
