/*
 * wait.h
 *-
 * Copyright (c) 2001, 2002, 2003 Corrado Santoro (csanto@diit.unict.it)
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Id: wait.h,v 1.1.1.1 2005/05/15 09:49:59 corrado Exp $
 */

#ifndef __WAIT_H
#define __WAIT_H

#include <kernel/ntypes.h>
#include <kernel/task.h>
#include <kernel/timer.h>

typedef struct wait_list {
  task_struct * task;
  sw_timer * timer_data;
  uint8 skip;
  struct wait_list * next;
} wait_list;

typedef wait_list * wait_chan;

typedef struct {
  int value;
  wait_chan wchan;
} t_semaphore;

typedef struct {
  wait_chan wchan;
  t_semaphore * sem;
} t_cond;

#define INIT_NEW_SEMAPHORE(n) { n, NULL }
#define INIT_NEW_SEMAPHORE0 INIT_NEW_SEMAPHORE(0)
#define NEW_SEMAPHORE(s,n) { s.value = n; s.wchan = NULL; }

#define INIT_NEW_COND       { NULL , NULL }
#define NEW_COND(c)         { c.wchan = NULL; c.sem = NULL; }

void thread_wakeup(wait_chan * wchan);
void thread_wakeup_all(wait_chan * wchan);
void thread_sleep(wait_chan * wchan);
void thread_sleep_and_start_scheduler (wait_chan * wchan);
int thread_timed_sleep (wait_chan * wchan, unsigned long timeout);
void semaphore_up(t_semaphore * sem);
void semaphore_down(t_semaphore * sem);
void cond_wait(t_cond * cond,t_semaphore * sem);
void cond_signal(t_cond * cond);

#endif
