/*
 * shell.c
 *-
 * Copyright (c) 2001, 2002, 2003 Corrado Santoro (csanto@diit.unict.it)
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Id: shell.c,v 1.1.1.1 2005/05/15 09:50:00 corrado Exp $
 */

#include <pthread.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <fcntl.h>
#include <sys/time.h>
#include <unistd.h>
#include <ctype.h>
#include <errno.h>
#include <signal.h>
#include <sys/ioctl.h>
#include <sys/resource.h>
#include <sys/wait.h>
#include <setjmp.h>
#include <pthread.h>

typedef int (*user_program)(int argc, char *argv[]);

#define MAX_USER_PROGRAMS 256


#define ADD_USER_PROGRAM(n,p) { int p (int, char **); add_program(n,p); }

struct
{
  char * pgm_name;
  user_program pgm_proc;
}
user_program_table [MAX_USER_PROGRAMS];
int user_program_table_size = 0;

jmp_buf _restart;

char * trim_string (char * s)
{
  while ( (isspace (*s)) && (*s != 0) )
    ++s;
  while ( (*s != 0) && (isspace (s[strlen(s)-1])) )
    --s;
  return s;
}

void add_program (char * pgm_name, user_program pgm_proc)
{
  user_program_table [user_program_table_size].pgm_name =
                            (char *)malloc (strlen(pgm_name) + 1);
  strcpy (user_program_table [user_program_table_size].pgm_name, pgm_name);
  user_program_table [user_program_table_size].pgm_proc = pgm_proc;
  user_program_table_size++;
}

user_program find_program (char * pgm_name)
{
  int i;

  if (!strcmp (pgm_name, "@"))
  {
    printf ("preparing for restart..\n");
    longjmp (_restart, 1); //only to test set/longjmp
  }

  for (i = 0;i < user_program_table_size;i++)
  {
    if (!strcmp (user_program_table [i].pgm_name, pgm_name))
      return user_program_table [i].pgm_proc;
  }
  return NULL;
}

#define MAX_ARGS  255

typedef struct
{
  int argc;
  char * argv[MAX_ARGS];
  user_program program;
} t_program_args;

void * run_program (void * p)
{
  t_program_args * args = (t_program_args *)p;

  args->program (args->argc, args->argv);
  exit (0);
  return NULL;
}

t_program_args args;

void exec_program (char * command, char * str_parameters)
{
  user_program  program;
  char  * blankposition;
  int status;
  pthread_t program_id;

  if (str_parameters == NULL)
    str_parameters = "";
  // search program in program list
  program = find_program (command);
  if (program == NULL)
  {
    fprintf (stderr, "%s: no such program\n", command);
    return;
  }

  args.argc = 1;
  args.argv[0] = command;
  args.program = program;

  // split arguments
  while (strlen (str_parameters) > 0)
  {
    blankposition = strchr (str_parameters,' ');
    if (blankposition == NULL)
    {
      args.argv [args.argc++] = str_parameters;
      break;
    }
    else
    {
      *blankposition = 0;
      args.argv [args.argc++] = trim_string (str_parameters);
      str_parameters = trim_string (blankposition + 1);
    }
  }
  // finally... execute the program
  pthread_create (&program_id, NULL, run_program, (void *)&args);
  wait (&status);
  /*
  if (fork () == 0)
  {
    program (argc, argv);
    exit (0);
  }
  else
  {
    int status;
    wait (&status);
  }
  */
}

void * sh (void * x)
{
  char  s[256], *cmd, *str_parameters;
  char  * blankposition;
  int  jmpval;

  #include "../usercommands/usertab.inc"

  jmpval = setjmp (_restart);
  if (jmpval == 1)
    printf("NUXI Shell restarting....\n");

  while (1)
  {
    printf("nuxi$ ");
    fgets(s,255,stdin);
    if (strlen(s) > 0)
    {
      blankposition = strchr (s,' ');
      if (blankposition == NULL)
      {
        cmd = (char *)malloc (strlen (s) + 1);
        strcpy (cmd, s);
        str_parameters = NULL;
      }
      else
      {
        *blankposition = 0;
        blankposition = trim_string (blankposition + 1);
        cmd = (char *)malloc (strlen (s) + 1);
        strcpy (cmd, s);
        if (*blankposition == 0)
          str_parameters = NULL;
        else
        {
          str_parameters = (char *)malloc (strlen (blankposition) + 1);
          strcpy (str_parameters, blankposition);
        }
      }
      exec_program (cmd, str_parameters);
      free (cmd);
      if (str_parameters != NULL)
        free (str_parameters);
    }
  }
}

