#
# NUXI Boot Code
# Copyright (c) 2001, 2002, 2003 Corrado Santoro (csanto@diit.unict.it)
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
# FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
# DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
# OR SERVICES, LOSS OF USE, DATA, OR PROFITS, OR BUSINESS INTERRUPTION)
# HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
# OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
# SUCH DAMAGE.
#
# $Id: boot.s,v 1.1.1.1 2005/05/15 09:50:01 corrado Exp $
#
# This code is loaded from BIOS at address 0000:7C00
#

.include "segments.s"

BOOTSEG = 0x9FE0
STACKSEG = 0x9F00
KERNEL_LOAD_SEG = 0x0

.global boot

        .org 0
        .code16
        .balign      0x08
boot:
        jmp next
sector_count: .word  0
sector: .byte        2
head:   .byte        0
track:  .byte        0
address: .word        kaddress
segment: .word        KERNEL_LOAD_SEG

next:
        movw    $0x07c0,%ax
        movw    %ax,%es
        movw    $bootmsg, %bx
        call    wstring
                                  # now move boot sector at BOOTSEG:0000H
        movw    $0x7c0,%ax        # boot sector is loaded at 07C0H:0000H
        movw    %ax,%ds           # thus load DS to access this area
        movw    $0,%si            # ds:si = loaded boot sector area
        movw    $BOOTSEG,%ax
        movw    %ax,%es
        movw    $0,%di            # es:di = destination area
        movw    $256,%cx          # word count
        cld
        rep
        movsw                     # relocate sector
        ljmp    $BOOTSEG,$go      # go to relocated code
go:
        movw    $STACKSEG,%ax
        movw    %ax,%ss
        movw    $0x0df0,%sp  #place stack at STACKSEG:0DF0H (before boot sector)
        call    writedot          # put a dot onto the screen to signal
                                  # the first boot stage

        movw    $BOOTSEG,%ax      # now boot sector is at BOOTSEG:0000H
        movw    %ax,%ds           # thus load DS to access this area
        movw    sector_count,%cx  # loop for the number of sectors
                                  # to read
do_read:
        pushw   %cx
        movw    segment, %ax      # prepare register for sector read
        movw    %ax, %es          # address (segment)
        movw    address, %bx      # address (offset)
        movb    $0x02, %ah        # service 2 (read sectors)
        movb    $0x01,%al         # sector count=1 sector
        movb    sector, %cl       # sector number
        movb    track,%ch         # track number
        movb    head,%dh          # head number
        movb    $0x0,%dl          # drive number (0=A:)
        int     $0x13             # read a sector of microkernel image
        jnc     read_ok

        movb    $0x45,%al         # if error, display 'E' and retry
        call    writechar
        popw    %cx
        jmp     do_read

read_ok:
        call    writedot          # sector loaded successfully
                                  # now go to next sector

        addw    $0x20, segment    # move forward segment by 32 
                                  # (sector size is 512 bytes)
        incb    sector
        cmpb    $19,sector         # are we at the end of track?
                                   # (1.44 MB floppy => 18 secs./track)
        jne     read_next
        movb    $1,sector
        incb    head               # end of track reached => next disk face
        cmpb    $2,head            # did we read both faces ?
        jne     read_next
        movb    $0,head            # both faces read, go to next track
        incb    track
read_next:
        popw    %cx
        loop    do_read            # loop to total number of sector to read

        movw    $0x03F2, %dx       # turn off drive motor
        xorb    %al, %al
        outb    %al, %dx
  
        call    writedot           # motor off successfully

        ljmp    $ksegment, $kaddress

# write a dot into the screen
writedot:
        movb    $0x2e,%al
writechar:
        movb    $0x0e,%ah
        movw    $0x0007,%bx
        int     $0x10
wstring_exit:
        ret

wstring:
        movb    %es:(%bx), %al
        cmpb    $0,%al
        jz      wstring_exit
        push    %bx
        call    writechar
        pop     %bx
        incw    %bx
        jmp     wstring

bootmsg:  .ascii "NUXI Boot Loader 0.3"
          .byte   0x0d, 0x0a
          .ascii "Loading System "
          .byte   0

        .org   510, 0
        .byte 0x55,0xAA
