/*
 * clist.c
 *-
 * Copyright (c) 2001, 2002, 2003 Corrado Santoro (csanto@diit.unict.it)
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Id: clist.c,v 1.1.1.1 2005/05/15 09:50:01 corrado Exp $
 */

#include <kernel/ntypes.h>
#include <kernel/kprintf.h>
#include <kernel/string.h>
#include <kernel/mem.h>
#include <kernel/mm.h>
#include <kernel/wait.h>
#include <kernel/clist.h>

t_clist * clist_new (void)
{
  t_clist * c;
  c = (t_clist *)kmalloc (sizeof (t_clist));
  if (c == NULL)  return NULL;
  NEW_SEMAPHORE (c->clist_mutex,1);
  NEW_COND (c->clist_empty);
  c->head = c->tail = NULL;
  c->size = 0;
  return c;
}


void clist_put (t_clist * c,char * buf,int len)
{
  t_cblock * b, * bnext;
  int i = 0;
  semaphore_down (&c->clist_mutex);
  while (i < len)
  {
    if (c->head == NULL)
    {
      b = (t_cblock *)kmalloc (sizeof (t_cblock));
      if (b == NULL) goto put_exit;
      b->next = NULL;
      b->head = b->tail = 0;
      c->head = c->tail = b;
    }
    b = c->tail;
    if (b->tail == CBLOCK_SIZE)
    {
      bnext = (t_cblock *)kmalloc (sizeof (t_cblock));
      if (bnext == NULL) goto put_exit;
      bnext->next = NULL;
      bnext->head = bnext->tail = 0;
      b->next = bnext;
      c->tail = bnext;
      b = bnext;
    }
    b->data[b->tail++] = buf[i++];
    c->size++;
  }
  cond_signal (&c->clist_empty);
put_exit:
  semaphore_up (&c->clist_mutex);
}

char clist_get (t_clist * c)
{
  t_cblock * b, * bnext;
  char  retval;
  semaphore_down (&c->clist_mutex);
  while (c->size == 0) cond_wait (&c->clist_empty, &c->clist_mutex);
  b = c->head;
  retval = b->data[b->head++];
  c->size--;
  if (b->head == CBLOCK_SIZE)
  {
    bnext = b->next;
    kfree (b);
    c->head = bnext;
  }
  semaphore_up (&c->clist_mutex);
  return retval;
}

void clist_delete (t_clist * c)
{
  t_cblock * b, * bnext;
  b = c->head;
  while (b != NULL)
  {
    bnext = b->next;
    kfree (b);
    b = bnext;
  }
  kfree (c);
}
