/*
 * driver.c
 *-
 * Copyright (c) 2001, 2002, 2003 Corrado Santoro (csanto@diit.unict.it)
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Id: driver.c,v 1.1.1.1 2005/05/15 09:50:02 corrado Exp $
 */

#include <kernel/driver.h>
#include <kernel/wait.h>
#include <kernel/mm.h>
#include <kernel/mem.h>
#include <kernel/string.h>
#include <kernel/kprintf.h>

t_file_drv_entry  * file_driver_list = NULL;
t_semaphore file_driver_sem = INIT_NEW_SEMAPHORE(1);

t_file_drv_entry * atomic_find_file_driver(char * name)
{
  t_file_drv_entry  * p;
  p = file_driver_list;
  while (p != NULL)
  {
    if (!strcmp(name,p->name))
      return p;
    p = p->next;
  }
  return NULL;
}

int find_file_driver(char * name,t_file_drv_entry **drv)
{
  t_file_drv_entry  * p;
  semaphore_down(&file_driver_sem);
  p = atomic_find_file_driver(name);
  *drv = p;
  semaphore_up(&file_driver_sem);
  return (p == NULL) ? 0 : 1;
}

int register_file_driver(char * name,t_file_drv_operations * ops)
{
  t_file_drv_entry  * p;
  semaphore_down(&file_driver_sem);
  if (atomic_find_file_driver(name) != NULL)
  {
    kprintf("register file driver `%s': driver exists\r\n",name);
    semaphore_up(&file_driver_sem);
    return 0;
  }
  p = (t_file_drv_entry *)kmalloc(sizeof(t_file_drv_entry));
  if (p == NULL)
  {
    kprintf("register file driver `%s': no mem space\r\n",name);
    semaphore_up(&file_driver_sem);
    return 0;
  }
  strcpy(p->name,name);
  kmemcpy((uint8*)&p->operations,(uint8*)ops,sizeof(t_file_drv_operations));
  p->next = file_driver_list;
  file_driver_list = p;
  semaphore_up(&file_driver_sem);
  return 1;
}

int unregister_file_driver(char * name)
{
  t_file_drv_entry  * p,*prev;
  semaphore_down(&file_driver_sem);
  prev = NULL;
  p = file_driver_list;
  while (p != NULL)
  {
    if (!strcmp(name,p->name))
    {
      if (prev == NULL)
      {
        file_driver_list = NULL;
        kfree(p);
        semaphore_up(&file_driver_sem);
        return 1;
      }
      else
      {
        prev->next = p->next;
        kfree(p);
        semaphore_up(&file_driver_sem);
        return 1;
      }
    }
    prev = p;
    p = p->next;
  }
  semaphore_up(&file_driver_sem);
  return 0;
}

// add your calls to driver initialization routines here

#include <kernel/console.h>

#define ADD_DRIVER(p) { void p##_init(void); p##_init (); }

void driver_init(void)
{
  kprintf("Initializing drivers: ");

  #include "drivertab.inc"

  kprintf("\r\nDrivers Initialized OK\r\n");
}
