/*
 * exception.c
 *-
 * Copyright (c) 2001, 2002, 2003 Corrado Santoro (csanto@diit.unict.it)
 * Copyright (c) 2002 Dario Russo (dariorusso@hotmail.com)
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Id: exception.c,v 1.1.1.1 2005/05/15 09:50:01 corrado Exp $
 */
#include <kernel/kprintf.h>
#include <kernel/asm.h>
#include <kernel/io.h>
#include <kernel/interrupt.h>
#include <kernel/kernel.h>
#include <kernel/task.h>
#include <kernel/exception.h>

void fpu_save_context(void)
{
	//kprintf("Save context not implemented yet!\r\n");
	asm("clts");
}

// FIXME! not all exceptions have the error_code
void general_protection_violation()
{
  register t_ex_regs * regs;
  int i;
  interrupts_enabled = 0;
  __cli();
  load_regs(regs);
	kprintf("EXCEPTION: General Protection Violation\r\n");
  kprintf("Task %ld, Exception Code = %08lx\r\n",
          current_task->tid, regs->error_code);
  dump_regs(regs);
  dump_stack();
  task_end (0);
}

void invalid_opcode()
{
  register t_ex_regs * regs;
  int i;
  interrupts_enabled = 0;
  __cli();
  load_regs(regs);
  kprintf("PANIC: Invalid Opcode (int 0x06)\r\n");
  dump_regs(regs);
  dump_stack();
  __halt();
}

void double_exception()
{
	kprintf("PANIC: Double Exception (int 0x08)\r\n");
  __halt();
}

void divide_error()
{
	kprintf("PANIC:divide error 0x00\r\n");
  __halt();
}

void debug_exception()
{
  register t_ex_regs * regs;
  int i;
  interrupts_enabled = 0;
  __cli();
  load_regs(regs);
  kprintf("PANIC: debug_exception\r\n");
  dump_regs(regs);
  dump_stack();
  __halt();
}

void non_maskable_interrupt()
{
  kprintf("PANIC:non_maskable_interrupt\r\n");
  __halt();
}

void breakpoint()
{
  kprintf("breakpoint exception\r\n");
  __halt();
}

void into_detected_overflow()
{
  kprintf("PANIC:into_detected_overflow\r\n");
  __halt();
}

void bound_range_excedeed()
{
  kprintf("PANIC:bound_range_excedeed\r\n");
  __halt();
}

void processor_extention_protection()
{
  kprintf("PANIC:processor_extention_protection\r\n");
  __halt();
}

void invalid_tss()
{
  kprintf("PANIC:invalid_tss\r\n");
  __halt();
}

void segment_not_present()
{
  kprintf("PANIC:segment_not_present\r\n");
  __halt();
}

void stack_fault ()
{
  kprintf("PANIC:stack_fault\r\n");
  __halt();
}

void page_fault()
{
  kprintf("PANIC:page_fault not implemented yet\r\n");
  __halt();
}

void  coprocessor_error()
{
  kprintf("PANIC:coprocessor_error\r\n");
  __halt();
}

inline void exception_init(void)
{
	kprintf("\r\nRegistering exception handlers...");
	register_interrupt(0x0,divide_error);
	register_interrupt(0x1,debug_exception);
	register_interrupt(0x2,non_maskable_interrupt);
	register_interrupt(0x3,breakpoint);
	register_interrupt(0x4,into_detected_overflow);
	register_interrupt(0x5,bound_range_excedeed);
	register_interrupt(0x6,invalid_opcode);
	register_interrupt(0x7,fpu_save_context);
	register_interrupt(0x8,double_exception);
	register_interrupt(0x9,processor_extention_protection);
	register_interrupt(0xA,invalid_tss);
	register_interrupt(0xB,segment_not_present);
	register_interrupt(0xC,stack_fault);
	register_interrupt(0xD,general_protection_violation);
	register_interrupt(0xE,page_fault);
	register_interrupt(0x10,coprocessor_error);
}
