/*
 * fcntl.c
 *-
 * Copyright (c) 2001, 2002, 2003 Corrado Santoro (csanto@diit.unict.it)
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Id: fcntl.c,v 1.1.1.1 2005/05/15 09:50:01 corrado Exp $
 */

#include <kernel/ntypes.h>
#include <kernel/task.h>
#include <kernel/errno.h>
#include <kernel/syscall.h>
#include <user/fcntl.h>

int open(const char * name,int mode)
{
  PREPARE_CALL(SYS_OPEN);
  PREPARE_PARAM(0,(uint32)name);
  PREPARE_PARAM(1,(uint32)mode);
  MAKE_CALL;
  return SYS_RETVAL;
}

long write(int fid,char * buf,long size)
{
  PREPARE_CALL(SYS_WRITE);
  PREPARE_PARAM(0,(uint32)fid);
  PREPARE_PARAM(1,(uint32)buf);
  PREPARE_PARAM(2,(uint32)size);
  MAKE_CALL;
  return SYS_RETVAL;
}

long read(int fid,char * buf,long size)
{
  PREPARE_CALL(SYS_READ);
  PREPARE_PARAM(0,(uint32)fid);
  PREPARE_PARAM(1,(uint32)buf);
  PREPARE_PARAM(2,(uint32)size);
  MAKE_CALL;
  return SYS_RETVAL;
}

long ioctl(int fid,int cmd,void * arg)
{
  PREPARE_CALL(SYS_IOCTL);
  PREPARE_PARAM(0,(uint32)fid);
  PREPARE_PARAM(1,(uint32)cmd);
  PREPARE_PARAM(2,(uint32)arg);
  MAKE_CALL;
  return SYS_RETVAL;
}

off_t lseek(int fid,off_t offset,int whence)
{
  PREPARE_CALL(SYS_LSEEK);
  PREPARE_PARAM(0,(uint32)fid);
  PREPARE_PARAM(1,(uint32)offset);
  PREPARE_PARAM(2,(uint32)whence);
  MAKE_CALL;
  return SYS_RETVAL;
}

int fstat (int fid, struct stat *sb)
{
  PREPARE_CALL(SYS_FSTAT);
  PREPARE_PARAM(0,(uint32)fid);
  PREPARE_PARAM(1,(uint32)sb);
  MAKE_CALL;
  return SYS_RETVAL;
}

int stat (char *path, struct stat *sb)
{
  int f, retval;
  f = open (path, O_RDONLY);
  if (f < 0) return f;
  retval = fstat (f, sb);
  close (f);
  return retval;
}

int close(int fid)
{
  PREPARE_CALL(SYS_CLOSE);
  PREPARE_PARAM(0,(uint32)fid);
  MAKE_CALL;
  return SYS_RETVAL;
}

