/*
 * filesystems.c
 *-
 * Copyright (c) 2001, 2002, 2003 Corrado Santoro (csanto@diit.unict.it)
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Id: filesystems.c,v 1.1.1.1 2005/05/15 09:50:01 corrado Exp $
 */

#include <kernel/asm.h>
#include <kernel/kprintf.h>
#include <kernel/string.h>
#include <kernel/wait.h>
#include <kernel/mm.h>
#include <kernel/mem.h>
#include <kernel/filesystems.h>
#include <kernel/driver.h>

t_file_sys_entry  * file_system_list = NULL;
t_semaphore file_system_sem = INIT_NEW_SEMAPHORE(1);

t_file_sys_entry * atomic_find_file_system (char * name)
{
  t_file_sys_entry  * p;
  p = file_system_list;
  while (p != NULL)
  {
    if (!strcmp(name,p->name))
      return p;
    p = p->next;
  }
  return NULL;
}

int find_file_system (char * name,t_file_sys_entry **drv)
{
  t_file_sys_entry  * p;
  semaphore_down (&file_system_sem);
  p = atomic_find_file_system (name);
  *drv = p;
  semaphore_up (&file_system_sem);
  return (p == NULL) ? 0 : 1;
}

int register_file_system (char * name,t_file_sys_operations * ops)
{
  t_file_sys_entry  * p;
  semaphore_down (&file_system_sem);
  if (atomic_find_file_system (name) != NULL)
  {
    kprintf ("register file system `%s': already exists\r\n",name);
    semaphore_up (&file_system_sem);
    return 0;
  }
  p = (t_file_sys_entry *)kmalloc (sizeof (t_file_sys_entry));
  if (p == NULL)
  {
    kprintf ("register file system `%s': no mem space\r\n", name);
    semaphore_up (&file_system_sem);
    return 0;
  }
  strcpy (p->name, name);
  kmemcpy ((uint8*)&p->operations,
           (uint8*)ops,
           sizeof (t_file_sys_operations));
  p->next = file_system_list;
  file_system_list = p;
  semaphore_up (&file_system_sem);
  return 1;
}

int unregister_file_system (char * name)
{
  t_file_sys_entry  * p,*prev;
  semaphore_down (&file_system_sem);
  prev = NULL;
  p = file_system_list;
  while (p != NULL)
  {
    if (!strcmp (name, p->name))
    {
      if (prev == NULL)
      {
        file_system_list = NULL;
        kfree (p);
        semaphore_up (&file_system_sem);
        return 1;
      }
      else
      {
        prev->next = p->next;
        kfree (p);
        semaphore_up (&file_system_sem);
        return 1;
      }
    }
    prev = p;
    p = p->next;
  }
  semaphore_up (&file_system_sem);
  return 0;
}

int _fs_open (file *f, char * name, int mode)
{
  return 0;
}

int32 _fs_read (file * f, char * buf, int32 len)
{
  return 0;
}

int32 _fs_write (file * f, char * buf, int32 len)
{
  return 0;
}

int _fs_close (file * f)
{
  return 0;
}

int _fs_ioctl (file *f, uint32 cmd, void * args)
{
  return 0;
}

int32 _fs_lseek (file *f, int32 offset, int whence)
{
  return 0;
}

t_file_drv_operations fsys_ops = {
  _fs_open,
  _fs_read,
  _fs_write,
  _fs_close,
  _fs_ioctl,
  _fs_lseek,
  NULL  // _fs_fstat
};

// add your calls to filesystem initialization routines here

//void sysdrv_init();
//void cfifo_init();
//void bfifo_init();
//void null_init();
//void ramdisk_init();

void filesystem_init (void)
{
  if (!register_file_driver("fs", &fsys_ops))
  {
    kprintf("Error registering filesystem driver");
    __cli();
    for (;;) ;
  }
  kprintf("FileSystem Driver registered OK\r\n");
  kprintf("Initializing filesystems: ");

  //console_init ();
  //sysdrv_init ();
  //cfifo_init ();
  //bfifo_init ();
  //ramdisk_init ();
  //null_init ();

  kprintf("\r\nFilesystems Initialized OK\r\n");
}
