/*
 * gdt.c
 *-
 * Copyright (c) 2001, 2002, 2003 Corrado Santoro (csanto@diit.unict.it)
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Id: gdt.c,v 1.1.1.1 2005/05/15 09:50:01 corrado Exp $
 */

#include <kernel/io.h>
#include <kernel/kprintf.h>
#include <kernel/interrupt.h>
#include <kernel/gdt.h>

extern gdt_entry  * _gdt;

void display_gdt(uint16 selector)
{
  uint16 * adr;
  selector >>= 3;
  adr = (uint16 *) &_gdt[selector];
  kprintf("%x %x %x %x\r\n",*adr,*(adr+1),*(adr+2),*(adr+3));
}

// create a TSS entry in the GDT at the specified selector
void gdt_make_tss(uint16 selector,
                  uint32 segment_base,uint32 segment_limit,uint16 dpl)
{
  selector >>= 3;
  _gdt[selector].limit_lo = segment_limit & 0xffff;
  _gdt[selector].base_lo  = segment_base & 0xffff;
  _gdt[selector].type     = ((segment_base >> 16) & 0xff) |
                            0x8900 | // present, type = 486 tss
                            (dpl << 13);
  _gdt[selector].base_limit_hi = ((segment_limit >> 16) & 0xf) |
                              ((segment_base >> 16) & 0xff00);
}

// create a data segment entry in the GDT at the specified selector
void gdt_make_data_segment(uint16 selector,
                  uint32 segment_base,uint32 segment_limit,uint16 dpl,
                  uint8 granul,uint8 writ,uint8 dir)
{
  selector >>= 3;
  _gdt[selector].limit_lo = segment_limit & 0xffff;
  _gdt[selector].base_lo  = segment_base & 0xffff;
  _gdt[selector].type     = ((segment_base >> 16) & 0xff) |
                            0x9000 | // present, data type
                            (writ << 9) |
                            (dir << 10) |
                            (dpl << 13);
  _gdt[selector].base_limit_hi = ((segment_limit >> 16) & 0xf) |
                              ((segment_base >> 16) & 0xff00) |
                              0x0400 | // big
                              (granul << 7);
}

// create a code segment entry in the GDT at the specified selector
void gdt_make_code_segment(uint16 selector,
                  uint32 segment_base,uint32 segment_limit,uint16 dpl,
                  uint8 granul,uint8 rd,uint8 conf)
{
  selector >>= 3;
  _gdt[selector].limit_lo = segment_limit & 0xffff;
  _gdt[selector].base_lo  = segment_base & 0xffff;
  _gdt[selector].type     = ((segment_base >> 16) & 0xff) |
                            0x9800 | // present, code type
                            (rd << 9) |
                            (conf << 10) |
                            (dpl << 13);
  _gdt[selector].base_limit_hi = ((segment_limit >> 16) & 0xf) |
                              ((segment_base >> 16) & 0xff00) |
                              0x0400 | // big
                              (granul << 7);
}

