/*
 * interrupt.c
 *-
 * Copyright (c) 2001, 2002, 2003 Corrado Santoro (csanto@diit.unict.it)
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Id: interrupt.c,v 1.1.1.1 2005/05/15 09:50:01 corrado Exp $
 */

#include <kernel/ntypes.h>
#include <kernel/io.h>
#include <kernel/asm.h>
#include <kernel/interrupt.h>
#include <kernel/kprintf.h>
#include <kernel/kernel.h>
#include <kernel/exception.h>

// the base handler for the first irq
#define IRQ_BASE  0x20
#define INTERRUPT_VECTORS 0x40

// the interrupt descriptor table is located at the beginning of memory
idt_entry * idt;
extern uint32 _idtr;
void _load_idtr();
uint8 interrupts_enabled = 0;

int_service_t  int_services[INTERRUPT_VECTORS];

extern uint32 _int00_handler;
extern uint32 _int01_handler;
extern uint32 _int02_handler;
extern uint32 _int03_handler;
extern uint32 _int04_handler;
extern uint32 _int05_handler;
extern uint32 _int06_handler;
extern uint32 _int07_handler;
extern uint32 _int08_handler;
extern uint32 _int09_handler;
extern uint32 _int0a_handler;
extern uint32 _int0b_handler;
extern uint32 _int0c_handler;
extern uint32 _int0d_handler;
extern uint32 _int0e_handler;
extern uint32 _int0f_handler;
extern uint32 _int10_handler;
extern uint32 _int11_handler;
extern uint32 _int12_handler;
extern uint32 _int13_handler;
extern uint32 _int14_handler;
extern uint32 _int15_handler;
extern uint32 _int16_handler;
extern uint32 _int17_handler;
extern uint32 _int18_handler;
extern uint32 _int19_handler;
extern uint32 _int1a_handler;
extern uint32 _int1b_handler;
extern uint32 _int1c_handler;
extern uint32 _int1d_handler;
extern uint32 _int1e_handler;
extern uint32 _int1f_handler;
extern uint32 _int20_handler;
extern uint32 _int21_handler;
extern uint32 _int22_handler;
extern uint32 _int23_handler;
extern uint32 _int24_handler;
extern uint32 _int25_handler;
extern uint32 _int26_handler;
extern uint32 _int27_handler;
extern uint32 _int28_handler;
extern uint32 _int29_handler;
extern uint32 _int2a_handler;
extern uint32 _int2b_handler;
extern uint32 _int2c_handler;
extern uint32 _int2d_handler;
extern uint32 _int2e_handler;
extern uint32 _int2f_handler;
extern uint32 _int30_handler;
extern uint32 _int31_handler;
extern uint32 _int32_handler;
extern uint32 _int33_handler;
extern uint32 _int34_handler;
extern uint32 _int35_handler;
extern uint32 _int36_handler;
extern uint32 _int37_handler;
extern uint32 _int38_handler;
extern uint32 _int39_handler;
extern uint32 _int3a_handler;
extern uint32 _int3b_handler;
extern uint32 _int3c_handler;
extern uint32 _int3d_handler;
extern uint32 _int3e_handler;
extern uint32 _int3f_handler;

/*
void disable(void)
{
  __cli();
}

void enable(void)
{
  if (interrupts_enabled)
    __sti();
}
*/

// setup an entry of the IDT as Interrupt gate
// Warning! You must enter this routine with interrupt disabled!!
void setup_idt_entry_int_gate(int int_no,uint32 * paddr,uint32 selector)
{
  uint32 addr = (uint32)paddr;
  idt = 0x0;
  idt[int_no].offset_lo = addr & 0xffff;
  idt[int_no].selector  = selector;
  idt[int_no].type      = 0x8e00; // type = 486 interrupt gate
  idt[int_no].offset_hi = addr >> 16;
}

// setup an entry of the IDT as Task Gate
// Warning! You must enter this routine with interrupt disabled!!
void setup_idt_entry_task_gate(int int_no,uint32 * paddr,uint32 selector)
{
  uint32 addr = (uint32)paddr;
  idt = 0x0;
  idt[int_no].offset_lo = addr & 0xffff;
  idt[int_no].selector  = selector;
  idt[int_no].type      = 0x8500; // type = 486 task gate
  idt[int_no].offset_hi = addr >> 16;
}

void display_idt(void)
{
  uint16 * adr;
  int i;
  adr = (uint16 *) 0x0;
  for (i = 0;i < 10 ;i++) {
    kprintf("%x %x %x %x\r\n",*adr,*(adr+1),*(adr+2),*(adr+3));
    adr+=4;
  }
}

// prepare circuitry and structures for interrupt handling
void interrupt_setup(void)
{
  int i;
  kprintf("Initializing interrupts units...");
  // first initialize 8259A-1 and -2
  outb(0x11,0x20); __SLOW_DOWN_IO;  // initialization sequence
  outb(IRQ_BASE,0x21); __SLOW_DOWN_IO;  // irq 0-7 located at 0x20-0x27
  outb(0x04,0x21); __SLOW_DOWN_IO;  // 8259-1 is master
  outb(0x01,0x21); __SLOW_DOWN_IO;  // 8086 mode

  outb(0x11,0xa0); __SLOW_DOWN_IO;
  outb(IRQ_BASE+8,0xa1); __SLOW_DOWN_IO;  // irq 8-15 located at 0x28-0x2f
  outb(0x02,0xa1); __SLOW_DOWN_IO;  // 8259-2 is slave
  outb(0x01,0xa1); __SLOW_DOWN_IO;  // 8086 mode

  outb(0xff,0xa1); __SLOW_DOWN_IO;  // mask off all interrupts
  outb(0xfb,0x21); __SLOW_DOWN_IO;  // mask off interrupts but irq2 (cascade)

  // now setup the internal interrupt jump table
  for (i = 0;i < INTERRUPT_VECTORS;i++)
    int_services[i] = NULL;  // no ISR for each interrupt

  // setup the handlers for each interrupt
  setup_idt_entry_int_gate(0x00,&_int00_handler,KERNEL_CODE_SELECTOR);
  setup_idt_entry_int_gate(0x01,&_int01_handler,KERNEL_CODE_SELECTOR);
  setup_idt_entry_int_gate(0x02,&_int02_handler,KERNEL_CODE_SELECTOR);
  setup_idt_entry_int_gate(0x03,&_int03_handler,KERNEL_CODE_SELECTOR);
  setup_idt_entry_int_gate(0x04,&_int04_handler,KERNEL_CODE_SELECTOR);
  setup_idt_entry_int_gate(0x05,&_int05_handler,KERNEL_CODE_SELECTOR);
  setup_idt_entry_int_gate(0x06,&_int06_handler,KERNEL_CODE_SELECTOR);
  setup_idt_entry_int_gate(0x07,&_int07_handler,KERNEL_CODE_SELECTOR);
  setup_idt_entry_int_gate(0x08,&_int08_handler,KERNEL_CODE_SELECTOR);
  setup_idt_entry_int_gate(0x09,&_int09_handler,KERNEL_CODE_SELECTOR);
  setup_idt_entry_int_gate(0x0a,&_int0a_handler,KERNEL_CODE_SELECTOR);
  setup_idt_entry_int_gate(0x0b,&_int0b_handler,KERNEL_CODE_SELECTOR);
  setup_idt_entry_int_gate(0x0c,&_int0c_handler,KERNEL_CODE_SELECTOR);
  setup_idt_entry_int_gate(0x0d,&_int0d_handler,KERNEL_CODE_SELECTOR);
  setup_idt_entry_int_gate(0x0e,&_int0e_handler,KERNEL_CODE_SELECTOR);
  setup_idt_entry_int_gate(0x0f,&_int0f_handler,KERNEL_CODE_SELECTOR);
  setup_idt_entry_int_gate(0x10,&_int10_handler,KERNEL_CODE_SELECTOR);
  setup_idt_entry_int_gate(0x11,&_int11_handler,KERNEL_CODE_SELECTOR);
  setup_idt_entry_int_gate(0x12,&_int12_handler,KERNEL_CODE_SELECTOR);
  setup_idt_entry_int_gate(0x13,&_int13_handler,KERNEL_CODE_SELECTOR);
  setup_idt_entry_int_gate(0x14,&_int14_handler,KERNEL_CODE_SELECTOR);
  setup_idt_entry_int_gate(0x15,&_int15_handler,KERNEL_CODE_SELECTOR);
  setup_idt_entry_int_gate(0x16,&_int16_handler,KERNEL_CODE_SELECTOR);
  setup_idt_entry_int_gate(0x17,&_int17_handler,KERNEL_CODE_SELECTOR);
  setup_idt_entry_int_gate(0x18,&_int18_handler,KERNEL_CODE_SELECTOR);
  setup_idt_entry_int_gate(0x19,&_int19_handler,KERNEL_CODE_SELECTOR);
  setup_idt_entry_int_gate(0x1a,&_int1a_handler,KERNEL_CODE_SELECTOR);
  setup_idt_entry_int_gate(0x1b,&_int1b_handler,KERNEL_CODE_SELECTOR);
  setup_idt_entry_int_gate(0x1c,&_int1c_handler,KERNEL_CODE_SELECTOR);
  setup_idt_entry_int_gate(0x1d,&_int1d_handler,KERNEL_CODE_SELECTOR);
  setup_idt_entry_int_gate(0x1e,&_int1e_handler,KERNEL_CODE_SELECTOR);
  setup_idt_entry_int_gate(0x1f,&_int1f_handler,KERNEL_CODE_SELECTOR);
  setup_idt_entry_int_gate(0x20,&_int20_handler,KERNEL_CODE_SELECTOR);
  setup_idt_entry_int_gate(0x21,&_int21_handler,KERNEL_CODE_SELECTOR);
  setup_idt_entry_int_gate(0x22,&_int22_handler,KERNEL_CODE_SELECTOR);
  setup_idt_entry_int_gate(0x23,&_int23_handler,KERNEL_CODE_SELECTOR);
  setup_idt_entry_int_gate(0x24,&_int24_handler,KERNEL_CODE_SELECTOR);
  setup_idt_entry_int_gate(0x25,&_int25_handler,KERNEL_CODE_SELECTOR);
  setup_idt_entry_int_gate(0x26,&_int26_handler,KERNEL_CODE_SELECTOR);
  setup_idt_entry_int_gate(0x27,&_int27_handler,KERNEL_CODE_SELECTOR);
  setup_idt_entry_int_gate(0x28,&_int28_handler,KERNEL_CODE_SELECTOR);
  setup_idt_entry_int_gate(0x29,&_int29_handler,KERNEL_CODE_SELECTOR);
  setup_idt_entry_int_gate(0x2a,&_int2a_handler,KERNEL_CODE_SELECTOR);
  setup_idt_entry_int_gate(0x2b,&_int2b_handler,KERNEL_CODE_SELECTOR);
  setup_idt_entry_int_gate(0x2c,&_int2c_handler,KERNEL_CODE_SELECTOR);
  setup_idt_entry_int_gate(0x2d,&_int2d_handler,KERNEL_CODE_SELECTOR);
  setup_idt_entry_int_gate(0x2e,&_int2e_handler,KERNEL_CODE_SELECTOR);
  setup_idt_entry_int_gate(0x2f,&_int2f_handler,KERNEL_CODE_SELECTOR);
  setup_idt_entry_int_gate(0x30,&_int30_handler,KERNEL_CODE_SELECTOR);
  setup_idt_entry_int_gate(0x31,&_int31_handler,KERNEL_CODE_SELECTOR);
  setup_idt_entry_int_gate(0x32,&_int32_handler,KERNEL_CODE_SELECTOR);
  setup_idt_entry_int_gate(0x33,&_int33_handler,KERNEL_CODE_SELECTOR);
  setup_idt_entry_int_gate(0x34,&_int34_handler,KERNEL_CODE_SELECTOR);
  setup_idt_entry_int_gate(0x35,&_int35_handler,KERNEL_CODE_SELECTOR);
  setup_idt_entry_int_gate(0x36,&_int36_handler,KERNEL_CODE_SELECTOR);
  setup_idt_entry_int_gate(0x37,&_int37_handler,KERNEL_CODE_SELECTOR);
  setup_idt_entry_int_gate(0x38,&_int38_handler,KERNEL_CODE_SELECTOR);
  setup_idt_entry_int_gate(0x39,&_int39_handler,KERNEL_CODE_SELECTOR);
  setup_idt_entry_int_gate(0x3a,&_int3a_handler,KERNEL_CODE_SELECTOR);
  setup_idt_entry_int_gate(0x3b,&_int3b_handler,KERNEL_CODE_SELECTOR);
  setup_idt_entry_int_gate(0x3c,&_int3c_handler,KERNEL_CODE_SELECTOR);
  setup_idt_entry_int_gate(0x3d,&_int3d_handler,KERNEL_CODE_SELECTOR);
  setup_idt_entry_int_gate(0x3e,&_int3e_handler,KERNEL_CODE_SELECTOR);
  setup_idt_entry_int_gate(0x3f,&_int3f_handler,KERNEL_CODE_SELECTOR);

  _load_idtr();

  __sti();
  interrupts_enabled = 1;
#ifdef  DEBUG_IDT
  display_idt();
#endif
  exception_init();
  kprintf("OK\r\n");
}

// handles a generic interrupt
// It is called by the _intxx_handler (see int.s) defined in the IDT
void interrupt_handler(uint32 int_no)
{
  /* If a ISR is defined for this service, call it */
  if (int_services[int_no] != NULL)
    int_services[int_no]();
  else {
    kprintf("Unhandled interrupt 0x%x", int_no);
    __cli();
    for (;;) ;
  }
}

// register a new ISR
int register_interrupt(int int_no,int_service_t routine)
{
  unsigned long flags;
  //__cli();  // disable interrupts
  __save_flags_cli (flags);
  // check whether the irq is already registered
  if (int_services[int_no] == NULL) {
    // register the ISR
    int_services[int_no] = routine;
    // re-enable interrupts
    __restore_flags (flags);
    //__sti();
    // return successfully
    return 1;
  }
  else {
    __restore_flags (flags);
    //__sti();
    return 0;
  }
}

// register a new ISR relevant to an IRQ
// enables also the relevant bit of the 8259
int register_irq(int irq,int_service_t routine)
{
  __cli();  // disable interrupts
  // check whether the irq is already registered
  if (int_services[irq+IRQ_BASE] == NULL) {
    // register the ISR
    int_services[irq+IRQ_BASE] = routine;
    // now enable interrupt mask register of 8259
    if (irq < 8) {
      // reset the relevant irq bit
      unsigned char mask = ~(1 << irq);
      outb(inb(0x21) & mask,0x21);
      outb(0x20,0x20);
    }
    else {
      // reset the relevant irq bit
      unsigned char mask = ~(1 << (irq-8));
      outb(inb(0xa1) & mask,0xa1);
      outb(0x20,0xa0);
    }
    // re-enable interrupts
    __sti();
    // return successfully
    return 1;
  }
  else {
    __sti();
    return 0;
  }
}

// re-register a new ISR relevant to an IRQ
int re_register_irq(int irq,int_service_t routine)
{
  __cli();  // disable interrupts
  // check whether the irq is already registered
  if (int_services[irq+IRQ_BASE] != NULL) {
    // register the ISR
    int_services[irq+IRQ_BASE] = routine;
    // re-enable interrupts
    if (irq < 8) {
      outb(0x20,0x20);
    }
    else {
      outb(0x20,0xa0);
    }
    __sti();
    // return successfully
    return 1;
  }
  else {
    register_irq(irq,routine);
    return 0;
  }
}
// unregister   ISR relevant to an IRQ
int unregister_irq(int irq)
{
  __cli();  // disable interrupts
  // check whether the irq is already registered
  if (int_services[irq+IRQ_BASE] != NULL) {
    // unregister the ISR
    int_services[irq+IRQ_BASE] = NULL;
    // now disable interrupt mask register of 8259
    if (irq < 8) {
      // set the relevant irq bit
      unsigned char mask = 1 << irq;
      outb(inb(0x21) | mask,0x21);
      outb(0x20,0x20);
    }
    else {
      // set the relevant irq bit
      unsigned char mask = 1 << (irq-8);
      outb(inb(0xa1) | mask,0xa1);
      outb(0x20,0xa0);
    }
    // re-enable interrupts
    __sti();
    // return successfully
    return 1;
  }
  else {
    __sti();
    return 0;
  }
}

