/*
 * kernel.c
 *-
 * Copyright (c) 2001, 2002, 2003 Corrado Santoro (csanto@diit.unict.it)
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Id: kernel.c,v 1.1.1.1 2005/05/15 09:50:01 corrado Exp $
 */

#include <kernel/io.h>
#include <kernel/asm.h>
#include <kernel/console.h>
#include <kernel/interrupt.h>
#include <kernel/timer.h>
#include <kernel/task.h>
#include <kernel/kprintf.h>
#include <kernel/mm.h>
#include <kernel/wait.h>
#include <kernel/exception.h>
#include <kernel/driver.h>
#include <kernel/filesystems.h>
#include <kernel/syscall.h>
#include <kernel/sysfs.h>
#include <kernel/init.h>
#include <kernel/scheduler.h>
#include <user/stdio.h>
#include <user/fcntl.h>

char * KERNEL_RELEASE = "0.2.0-1";

void * machine_memory_size = 0;

void memory_test(void)
{
  // Start memtest from 0x100000 (1 MB)
  // Declare as 'volatile' otherwise assignment and comparison with 0x1234
  // does not work due to compiler optimization
  volatile uint16 * memaddr = (uint16 *)0x100000;
  while (memaddr != 0x0) {
    uint32 l_addr = (uint32)memaddr;
    l_addr >>= 20;
    kprintf("\rTesting Memory: %d MBytes",(int)l_addr);
    *memaddr = 0x1234;
    if (*memaddr != 0x1234)
      break;
    memaddr += 0x80000; // goto next megabyte
  }
  kprintf(" Found OK\r\n");
  machine_memory_size = (void *)memaddr;
}

void a20_enable(void)
{
  kprintf ("Enabling A20...");
  while ((inb(0x64) & 2) != 0);
  outb(0xd1,0x64);
  __SLOW_DOWN_IO;
  while ((inb(0x64) & 2) != 0);
  outb(0xdf,0x60);
  __SLOW_DOWN_IO;
  outb(inb(0x92) | 0x02,0x92); // fast "A20" version
  __SLOW_DOWN_IO;
  __SLOW_DOWN_IO;
  __SLOW_DOWN_IO;
  __SLOW_DOWN_IO;
  kprintf ("OK\r\n");
}

void reset_coprocessor(void)
{
  kprintf ("Resetting coprocessor...");
  outb(0xf0,0x0);
  __SLOW_DOWN_IO;
  outb(0xf1,0x0);
  __SLOW_DOWN_IO;

  asm ("finit;"
	  "clts;"
	  "mov %cr0,%eax;"
	  "and $0xfffffff1,%eax;"
	  "mov %eax,%cr0"
	  "");

  kprintf ("OK\r\n");
}

void nibble_print(uint32 x)
{
  kprintf("%c%c%c%c",
      x & 0xff , (x >> 8) & 0xff, (x >> 16) & 0xff, (x >> 24) & 0xff);
}

void cpuid_pentium(void)
{
  uint32  eax,ebx,ecx,edx;
  asm ("movl $0x0,%%eax;"
      "cpuid;"
      "movl %%eax,%0;"
      "movl %%ebx,%1;"
      "movl %%ecx,%2;"
      "movl %%edx,%3" : "=m" (eax) , "=m"  (ebx) , "=m" (ecx) , "=m" (edx));
  kprintf("CPU Type is `");
  nibble_print(ebx);
  nibble_print(edx);
  nibble_print(ecx);
  asm ("movl $0x1,%%eax;"
      "cpuid;"
      "movl %%eax,%0;"
      "movl %%edx,%1" : "=m" (eax) , "=m" (edx));
  kprintf("' Type %d, Family %d, Model %d, Stepping ID %d\r\n",
            (eax >> 12) & 3, (eax >> 8) & 0xf, (eax >> 4) & 0xf, eax & 0xf);
}

extern short _cpu_type;

void cpuid (void)
{
  switch (_cpu_type)
  {
    case 3:
      kprintf ("CPU Type is 'i386' or compatible\n\r");
      break;
    case 4:
      kprintf ("CPU Type is 'i486' or compatible\n\r");
      break;
    case 5:
      cpuid_pentium ();
      break;
    default:
      kprintf ("CPU Type is 'unknown'\n\r");
      break;
  }
}

/*
void user_main(void);

void * user_task(void * x)
{
  user_main();
  kprintf("Main User Task is terminated\r\n");
  task_end();
  return NULL;
}
*/

void kbanner(void)
{
  kprintf("               _      _     _      _ \r\n");
  kprintf("              | |    | |   | |    | |   \r\n");
  kprintf("              | |    | |   | |    | |    _\r\n");
  kprintf("              | |    | |    \\ \\  / /    |_|\r\n");
  kprintf("   _______    | |    | |     \\ \\/ /      _\r\n");
  kprintf("  /  ___  |   | |    | |     / /\\ \\     | |\r\n");
  kprintf("  | |   | |   | |    | |    / /  \\ \\    | |\r\n");
  kprintf("  | |   | |   | |____| |   | |    | |   | |\r\n");
  kprintf("  |_|   |_|   \\________|   |_|    |_|   |_|  %s started\r\n\n",
                                    KERNEL_RELEASE);
  kprintf("Copyright (C) 2001,2002,2003 Corrado Santoro (csanto@diit.unict.it)\r\n");
  kprintf("Copyright (C) 2001,2002,2003 the NUXI Team\r\n");
}

void main_kernel_task(void * x)
{
  kprintf ("Main Task Started OK\r\n");
  init_timer ();
  driver_init ();
  filesystem_init ();
  syscall_init ();
  cpuid ();
  kbanner ();

  // opening standard io
  if (open("con:",O_WRONLY) < 0)
  {
    kprintf("PANIC! Unable to open standard output\n\r");
    __halt ();
  }
  if (open("con:",O_RDONLY) < 0)
  {
    kprintf("PANIC! Unable to open standard input\n\r");
    __halt ();
  }
  if (open("con:",O_WRONLY) < 0)
  {
    kprintf("PANIC! Unable to open standard err\n\r");
    __halt ();
  }
  printf ("Starting 'init'...\r\n");
  #include "../init/inittab.inc"
  init_proc ();
  // now enter in the IDLE loop!
  for (;;) asm ("hlt");
}

void kernel_main()
{
  console_base_init();
  kprintf ("\r\nNUXI release %s loaded.\r\n",KERNEL_RELEASE);
  kprintf ("Copyright (C) 2001,2002 Corrado Santoro (csanto@diit.unict.it).\r\n");
  kprintf ("Compiled on " __DATE__ "\r\n");
  kprintf ("Booting the kernel.\r\n");
  a20_enable();
  memory_test();
  reset_coprocessor();
  interrupt_setup();
  kprintf("Preparing Memory Manager...");
  // hummm.. we consider heap memory starting from second megabyte
  // indeed we should start from the end of kernel (how to compute it?)
  mm_init(0x100000,(uint32)machine_memory_size);
  kprintf("OK\r\n");
  kprintf("Setup initial scheduler: round-robin...\r\n");
  set_scheduler (sched_rr_with_priority);
  kprintf("Preparing Task Structures...\r\n");
  task_add(NULL,main_kernel_task,NULL);
  // never return
  __cli();
  kprintf("PANIC! Unreachable point reached!\r\n");
  __halt();
}

unsigned int __divdi3(unsigned int a, unsigned int b)
{
  return a / b;
}

unsigned int __moddi3(unsigned int a, unsigned int b)
{
  return a % b;
}
