/*
 * mem.c
 *-
 * Copyright (c) 2001, 2002, 2003 Corrado Santoro (csanto@diit.unict.it)
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Id: mem.c,v 1.1.1.1 2005/05/15 09:50:01 corrado Exp $
 */

#include <kernel/ntypes.h>
#include <kernel/asm.h>
#include <kernel/mem.h>


uint8 * kmemcpy (uint8 * dst, uint8 * src, int size)
{
#ifdef __NOASSEMBLER
  uint8 * temp = dst;
  while (size > 0) {
    *dst = *src;
    dst++;
    src++;
    size--;
  }
  return temp;
#else
  /* this assember code (thanks to Dario Russo) requires
     that DS and ES have the same value */
  asm("pusha\n"
      "movl %0,%%esi\n"
      "movl %1,%%ecx\n"
      "movl %2,%%edi\n"
      "cld\n"
      "rep\nmovsb\n"
      "popa":: "m" (src), "m" (size), "m" (dst) : "esi", "edi", "ecx" );
  return dst;
#endif
}

int kmemcmp (uint8 * s1, uint8 * s2, int size)
{
  while ( (*s1 == *s2) && (size > 0))
  {
    ++s1;++s2; --size;
  }
  if (size == 0)
    return size;
  else
    return (*s1 - *s2);
}

// here size is in bytes
uint8 * kmemset (uint8 * dst, uint8 val, int size)
{
  --size;
  while (size >= 0)
    dst[size--] = val;
  return dst;
}

// here size is in word
uint16 * kmemsetw (uint16 * dst, uint16 val, int size)
{
  --size;
  while (size >= 0)
    dst[size--] = val;
  return dst;
}
