#include <kernel/ntypes.h>
#include <kernel/io.h>
#include <kernel/serial.h>
#include <kernel/asm.h>
#include <kernel/interrupt.h>
#include <kernel/mem.h>
#include <kernel/driver.h>
#include <kernel/kprintf.h>
#include <kernel/wait.h>

#define PORT1 0x3F8 //COM1
#define ISR_CYCLE_LIMIT 20 

int _serial_open(file * f,char * name,int mode)
{
  return 0;
}

wait_chan read_wchan=NULL;
wait_chan write_wchan=NULL;

#define BUFFER_LEN 128

struct {
  int num,head,tail;
  uint8 data[BUFFER_LEN];
} read_buffer = { 0,0,0,}; //coda circolare di lettura

void add_queue(uint8 c) {
  if(read_buffer.num < BUFFER_LEN) {
    read_buffer.data[read_buffer.head]  = c;
    read_buffer.head = (read_buffer.head + 1) % BUFFER_LEN;
    read_buffer.num++;
  }
} //funzione per inserire in testa un carattere  

int flag = 1;

void  serial_int(void)
{
  int counter=0;
  uint8 status,iid;
  iid = inb(PORT1 + 2); //per controllare se ci sono interrupt pending
  
  kprintf("\nInterrupt");
  do { //polling per controllare se ci sono interrupt pending
    
    status = inb(PORT1 + 5); //line status register
    //leggendo si resetta l'interrupt pending
    
    if (status & 0x01) { //received data available
      add_queue(inb(PORT1));
      thread_wakeup(&read_wchan);
      kprintf(" read\n");
    } else

    if (status & 0x40) {//trasmit register empty
      kprintf("  write\n");
      flag = 1;
      thread_wakeup(&write_wchan);
    }

    if(counter++ > ISR_CYCLE_LIMIT) { //per evitare di ciclare all'infinito
      kprintf("checking for interrupts broken");
      break;
    }

  } while  (iid & 0x01); //il ciclo ricomincia se ci sono interrupt pending

  outb(0x20,0x20);//resetta il pic 
}

int32 _serial_read(file * f,char * buf,int32 len)
{
  uint8  data;
  int count = 0;
  while (len > 0)
  {
    while (read_buffer.num == 0) thread_sleep(&read_wchan); //se il buffer e' vuoto il processo si mette a dormire nella wait queue
    data = read_buffer.data[read_buffer.tail];
    read_buffer.tail = (read_buffer.tail + 1) % BUFFER_LEN;
    --read_buffer.num;
    *buf++ = data;
    ++count;
    --len;
  }
  return count;
}

int32 _serial_write(file * f,char * buf,int32 len)
{
  uint8  data;
  int count = 0;
  while (len > 0)
  {
    if (flag) {//aspetta che ci sia l'interrupt che permetta di scrivere nel registro
      kprintf("\nmah%c\n",*buf);
      outb(*buf++, PORT1);
      ++count;
      --len;
      flag = 0;
    } else thread_sleep(&write_wchan);
  }
  return count;
}

int _serial_close(file * f)
{
  return 0;
}

t_file_drv_operations serial_ops = {
  _serial_open,
  _serial_read,
  _serial_write,
  _serial_close,
  NULL    // ioctl
};

void serial_init(void)
{
  if (!register_file_driver("ser",&serial_ops))
  {
    kprintf("Error registering serial driver");
    __cli();
    for (;;) ;
  }
  serial_set();
  kprintf("NUXI serial driver installed.\r\n");
}

void serial_set(void)
{
  outb(0x00, PORT1 + 1); //turn off interrupt
  register_irq(4, serial_int);
  outb(0x80, PORT1 + 3); //set DLAB on
  outb(0x03, PORT1 + 0); //set baud rate - divisor latch low byte
  outb(0x00, PORT1 + 1); //set baud rate - divisor latch high byte
  outb(0x03, PORT1 + 3); //8 bits no parity 1 stop bit
  outb(0xC7, PORT1 + 2); //FIFO control register
  outb(0x0B, PORT1 + 4); //turn on DTR, RTS and OUT2, Loopback
  //outb(inb(0x21) & 0xEF, 0x21);
  outb(0x03, PORT1 + 1); //enable interrupt 
}












