#
# setjmp/longjmp NUXI implementation
# -
# Copyright (c) 2001, 2002, 2003 Corrado Santoro (csanto@diit.unict.it)
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
# FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
# DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
# OR SERVICES, LOSS OF USE, DATA, OR PROFITS, OR BUSINESS INTERRUPTION)
# HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
# OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
# SUCH DAMAGE.
#
# $Id: setlongjump.s,v 1.1.1.1 2005/05/15 09:50:01 corrado Exp $
#

# This version of set/longjmp assumes that the caller saved registers
# for GCC/386 are ESI, EDI and EBX

        ESI_OFFSET = 0
        EDI_OFFSET = 4
        EBX_OFFSET = 8
        EBP_OFFSET = 12
        ESP_OFFSET = 16
        EIP_OFFSET = 20

        .global setjmp
        .global longjmp

        .code32
#
# setjmp (jmp_buf target)
setjmp:
        pushl   %ebp
        movl    %esp, %ebp
#
#                  Stack composition   EBP Offset
#                +------------------+
# esp = ebp ---> |     old_ebp      |  + 0
#                +------------------+
#                |<<return address>>|  + 4
#                +------------------+
#                |      target      |  + 8
#                +------------------+
#
        movl    0x8(%ebp), %eax         # get jmp_buf
        movl    %esi, ESI_OFFSET(%eax)  # save ESI
        movl    %edi, EDI_OFFSET(%eax)  # save EDI
        movl    %ebx, EBX_OFFSET(%eax)  # save EBX
        movl    %esp, ESP_OFFSET(%eax)  # save ESP

        # now get the stack structure

        movl    (%ebp), %ecx            # save old EBP
        movl    %ecx, EBP_OFFSET(%eax)
        movl    0x4(%ebp), %ecx         # save EIP
        movl    %ecx, EIP_OFFSET(%eax)

        movl    $0, %eax                 # this is the return value
        leave
        ret

#
# longjmp (jmp_buf target, int val)
# First 'val' is pushed, then 'target'
longjmp:
        pushl   %ebp
        movl    %esp, %ebp
#
#                  Stack composition   EBP Offset
#                +------------------+
# esp = ebp ---> |     old_ebp      |  + 0
#                +------------------+
#                |<<return address>>|  + 4
#                +------------------+
#                |      target      |  + 8
#                +------------------+
#                |       val        |  + 12
#                +------------------+
#
        movl    0x8(%ebp), %eax         # get jmp_buf
        movl    0xc(%ebp), %edx        # get val
        movl    ESI_OFFSET(%eax), %esi  # restore ESI
        movl    EDI_OFFSET(%eax), %edi  # restore EDI
        movl    EBX_OFFSET(%eax), %ebx  # restore EBX
        movl    ESP_OFFSET(%eax), %esp  # restore ESP

        # rebuild stack structure
        movl    %esp, %ebp
        movl    EBP_OFFSET(%eax), %ecx  # restore EBP
        movl    %ecx, (%ebp)            # put it onto the stack
        movl    EIP_OFFSET(%eax), %ecx  # restore EIP
        movl    %ecx, 0x4(%ebp)         # put it onto the stack

        movl    %edx, %eax              # this is the return value
        popl    %ebp
        ret

        .end
