#
# NUXI Startup Code
# Copyright (c) 2001, 2002, 2003 Corrado Santoro (csanto@diit.unict.it)
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
# FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
# DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
# OR SERVICES, LOSS OF USE, DATA, OR PROFITS, OR BUSINESS INTERRUPTION)
# HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
# OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
# SUCH DAMAGE.
#
# $Id: start.s,v 1.1.1.1 2005/05/15 09:50:01 corrado Exp $
#

.include "segments.s"

.global _gdt
.global start
.global _start
.global _load_idtr
.global _dummy_iret
.global _cpu_type

cputype_386 = 3
cputype_486 = 4
cputype_586 = 5

        .code16


  .balign  0x08
start:
_start:
        call    writedot  # jump to start code successfully

        movw    %cs,%ax    # setup DS equal to CS
        movw    %ax,%ds

        cli                 # disable interrupts
        lgdt    gdtr    # load global descriptor table

        call    writedot  # gdt setup OK

# OK. Set Bit0 of Machine Status Word --> Set Processor in Protected Mode
        movw    $0x1,%ax
        lmsw    %ax
# force flushing of instruction cache
        jmp     flush
flush:
# NOW IN PROTECTED MODE. Perform long jump to kernel startup code
#       jmpi    0x10,pm
        .byte   0x66,0xea
        .long   k_start
        .word   code

# ----------------------------------------
# subroutines for 16bit mode
        .balign 0x08
# write a dot into the screen
writedot:
        movb    $0x2e,%al
writech:
        movb    $0x0e,%ah
        movw    $0x0007,%bx
        int     $0x10
        ret

        .code32
k_start:
        movw    $data, %ax
        movw    %ax, %ds
        movw    %ax, %es
        movw    %ax, %fs
        movw    %ax, %gs
        movw    %ax, %ss
        movl    $0x09fff0,%eax
        movl    %eax,%esp

# check cpu type (from Intel documentation)
# check if AC bit of EFLAGS (bit #18) can be changed,
# if not the CPU is an i386
        movw    $cputype_386, %ax
        movw    %ax, _cpu_type
        pushfl
        popl    %eax
        movl    %eax,%ecx
        xorl    $0x40000,%eax
        pushl   %eax
        popfl
        pushfl
        popl    %eax
        xorl    %ecx,%eax
        andl    $0x40000,%eax
        je      cpu_detected

# check the presence of CPUID instruction (bit #21 of EFLAGS)
        movw    $cputype_486, %ax
        movw    %ax, _cpu_type
        movl    %ecx,%eax
        xorl    $0x200000,%eax
        pushl   %eax
        popfl
        pushfl
        popl    %eax
        xorl    %ecx,%eax
        andl    $0x200000,%eax
        je      cpu_detected

        movw    $cputype_586, %ax
        movw    %ax, _cpu_type
cpu_detected:
        
# booting the kernel main function (in kernel.c)
        cli
        jmp     kernel_main

        .balign 0x08
_cpu_type:  .word 0x0000

        .set    gdt_size,16

        .balign  0x08
        .word  0x0000
gdtr:   .word  (gdt_size * 8)-1
        .long  gdt

        .balign  0x08
gdt:
        .word  0x0000, 0x0000, 0x0000, 0x0000
        .word  0xFFFF, 0x0000, 0x9A00, 0x00CF
        .word  0xFFFF, 0x0000, 0x9200, 0x00CF
        .rept gdt_size-3
        .word  0x0000, 0x0000, 0x0000, 0x0000
        .endr

        .data
_gdt:
        .long gdt
        .end
