/*
 * stdlib.c
 *-
 * Copyright (c) 2001, 2002, 2003 Corrado Santoro (csanto@diit.unict.it)
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Id: stdlib.c,v 1.1.1.1 2005/05/15 09:50:01 corrado Exp $
 */

#include <user/ctype.h>
#include <kernel/ntypes.h>
#include <kernel/mm.h>
#include <user/signal.h>
#include <user/unistd.h>

long int strtol(const char *nptr,char **endptr,int base)
{
  long retval = 0;
  int  sign = 1;
  while ((*nptr != 0) && (isspace(*nptr)))
    ++nptr;
  if (*nptr == '-') { sign = -1;++nptr; }
  while (*nptr != 0)
  {
    int c;
    c = toupper(*nptr);
    if (isdigit(c))
      retval = (retval * base) + (c - '0');
    else if ( isalpha(c) && (base > 10) && ( (c - 'A' + 10) < base))
      retval = (retval * base) + (c - 'A' + 10);
    else break;
    nptr++;
  }
  if (endptr != NULL)
  {
    *endptr = (char *)nptr;
  }
  return retval*sign;
}

int atoi(char * s)
{
  char  *x;
  return strtol(s,&x,10);
}

long atol(char * s)
{
  char  *x;
  return strtol(s,&x,10);
}

void * malloc (int size)
{
  return kmalloc (size);
}

void free (void * ptr)
{
  kfree (ptr);
}

void abort (void)
{
  kill (getpid (), SIGABRT);
}
