/*
 * string.c
 *-
 * Copyright (c) 2001, 2002, 2003 Corrado Santoro (csanto@diit.unict.it)
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Id: string.c,v 1.1.1.1 2005/05/15 09:50:01 corrado Exp $
 */

#include <kernel/ntypes.h>
#include <kernel/string.h>
#include <kernel/mem.h>
#include <kernel/basic_printf.h>
#include <kernel/conv.h>
#include <kernel/string.h>

int strlen(const char * s)
{
  int len = 0;
  while (*s) {
    ++len;++s;
  }
  return len;
}

char * strchr(char * s,char c)
{
  while (*s != 0)
  {
    if (*s == c) return s;
    ++s;
  }
  return NULL;
}

char * strcpy(char * dst,const char * src)
{
  return (char *)kmemcpy((uint8*)dst,(uint8*)src,strlen(src)+1);
}

char * strncpy(char * dst,const char * src,int n)
{
  return (char *)kmemcpy((uint8*)dst,(uint8*)src,n);
}

char * strcat(char * dst,char * src)
{
  return (char *)kmemcpy((uint8*)(dst+strlen(dst)),(uint8*)src,strlen(src)+1);
}

int strcmp(const char * s1, const char *s2)
{
  while ( (*s1 == *s2) && (*s1 != 0) && (*s2 != 0))
  {
    ++s1;++s2;
  }
  if ((*s1 == 0) && (*s2 == 0))
    return 0; // s1 = s2
  if (*s1 == 0)
    return -1;  // s1 < s2
  if (*s2 == 0)
    return 1;  // s1 > s2
  return ( (*s1 > *s2) ? 1 : -1);
}

int strncmp (const char * s1,const char *s2, int n)
{
  while ( (*s1 == *s2) && (*s1 != 0) && (*s2 != 0) && (n > 0))
  {
    ++s1;++s2;--n;
  }
  if (n == 0)
    return 0; // s1 = s2
  if ((*s1 == 0) && (*s2 == 0))
    return 0; // s1 = s2
  if (*s1 == 0)
    return -1;  // s1 < s2
  if (*s2 == 0)
    return 1;  // s1 > s2
  return ( (*s1 > *s2) ? 1 : -1);
}

void * memcpy(void * dest,void * src,int size)
{
  return kmemcpy((uint8*)dest,(uint8*)src,size);
}

int memcmp(const void * s1,const void * s2,int size)
{
  return kmemcmp((uint8*)s1,(uint8*)s2,size);
}

void * memset(void * s,int c,int size)
{
  return kmemset((uint8*)s,c,size);
}
