/*
 * sys_process.c
 *-
 * Copyright (c) 2001, 2002, 2003 Corrado Santoro (csanto@diit.unict.it)
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Id: sys_process.c,v 1.1.1.1 2005/05/15 09:50:01 corrado Exp $
 */

#include <kernel/ntypes.h>
#include <kernel/asm.h>
#include <kernel/task.h>
#include <kernel/wait.h>
#include <kernel/sys_unistd.h>
#include <kernel/errno.h>

extern wait_chan wait_sys_call;

pid_t sys_fork (void)
{
  /*
  task_fork_prepare (current_task, NULL);
  __cli ();
  current_task->fork_request = 1;
  asm ( "int $0x20" ::); // do scheduler
  __sti ();
  return current_task->fork_ret_val;
  */
  return 0;
}

void sys_exit(int status)
{
  task_end (status);
}

pid_t sys_wait (int * status)
{
  task_struct * t;
  *status = 0;
  for (;;)
  {
    LOCK_SCHEDULER;
    t = task_find_by_parent (current_task->tid);
    if (t == NULL)
    {
      UNLOCK_SCHEDULER;
      errno = ECHILD;
      return -1;
    }
    if (t->state == TASK_ENDING)
    {
      int tid;
      tid = t->tid;
      *status = t->exit_code;
      task_remove (t);
      UNLOCK_SCHEDULER;
      return tid;
    }
    else
    {
      thread_sleep_and_start_scheduler (&wait_sys_call);
    }
  }
}

pid_t sys_waitpid (pid_t wpid, uint32 * status, int options)
{
  task_struct * t;
  *status = 0;
  for (;;)
  {
    LOCK_SCHEDULER;
    t = task_find (wpid);
    if (t == NULL)
    {
      UNLOCK_SCHEDULER;
      errno = ECHILD;
      return -1;
    }
    if (t->p_tid != current_task->tid)
    {
      UNLOCK_SCHEDULER;
      errno = ECHILD;
      return -1;
    }
    if (t->state == TASK_ENDING)
    {
      int tid;
      tid = t->tid;
      *status = t->exit_code;
      task_remove (t);
      UNLOCK_SCHEDULER;
      return tid;
    }
    else
    {
      thread_sleep_and_start_scheduler (&wait_sys_call);
    }
  }
}

pid_t sys_getpid(void)
{
  return current_task->tid;
}

pid_t sys_getppid(void)
{
  return current_task->p_tid;
}
