/*
 * sys_pthread.c
 * NUXI POSIX THREAD IMPLEMENTATION (kernel side)
 *-
 * Copyright (c) 2001, 2002, 2003 Corrado Santoro (csanto@diit.unict.it)
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Id: sys_pthread.c,v 1.1.1.1 2005/05/15 09:50:01 corrado Exp $
 */

#include <kernel/task.h>
#include <kernel/kprintf.h>
#include <kernel/sys_pthread.h>
#include <kernel/sys_unistd.h>
#include <kernel/errno.h>

typedef struct {
  void * (*startup)(void *);
  void * arg;
  task_struct * parent;
} thread_startup;

void * thread_start(void * func)
{
  int i;
  thread_startup st = *(thread_startup *) func;
  // copy file descriptors of parent task
  // and increment the reference count of each open file
  for (i = 0; i < MAX_FILES;i++)
  {
    current_task->files [i] = st.parent->files [i];
    if (current_task->files[i] != NULL)
      current_task->files[i]->ref_count++;
  }
  task_end( (uint32) st.startup(st.arg));
  return NULL;
}

int sys_pthread_create (pthread_t * pt,pthread_attr_t * pa,
                        void * (*start_routine)(void*),void * arg)
{
  uint32 th_id;
  thread_startup st;
  st.startup = start_routine;
  st.arg = arg;
  st.parent = current_task;
  task_add(&th_id,(startup_proc)thread_start,&st);
  pt->th_id = th_id;
  return 0;
}

int sys_pthread_join (pthread_t * pt, void ** value_ptr)
{
  int ret_value;
  uint32 status;
  ret_value = sys_waitpid (pt->th_id, &status, 0);
  if (ret_value < 0)
  {
    switch (errno)
    {
      case ECHILD:
        return ESRCH;
      default:
        return EINVAL;
    }
  }
  else
  {
    *value_ptr = (void *)status;
    return 0;
  }
}
