/*
 * syscall.c
 *-
 * Copyright (c) 2001, 2002, 2003 Corrado Santoro (csanto@diit.unict.it)
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Id: syscall.c,v 1.1.1.1 2005/05/15 09:50:01 corrado Exp $
 */

#include <kernel/kprintf.h>
#include <kernel/asm.h>
#include <kernel/io.h>
#include <kernel/interrupt.h>
#include <kernel/kernel.h>
#include <kernel/task.h>
#include <kernel/syscall.h>
#include <kernel/timer.h>
#include <kernel/stat.h>
#include <kernel/errno.h>
#include <kernel/sys_unistd.h>
#include <kernel/sys_pthread.h>

void syscall()
{
  int syscall_no;
  uint32  return_value;
  uint32  *params;
  //kprintf("System call %d\n",current_task->syscall.syscall_no);
  syscall_no = current_task->syscall.syscall_no;
  params = current_task->syscall.params;
  //__sti();
  switch (syscall_no)
  {
    /* filing system calls */
    case SYS_OPEN:
      return_value = sys_open((char *)params[0],(int)params[1]);
      break;
    case SYS_READ:
      return_value = sys_read((int)params[0],(char *)params[1],(long)params[2]);
      break;
    case SYS_WRITE:
      return_value =
            sys_write((int)params[0],(char *)params[1],(long)params[2]);
      break;
    case SYS_IOCTL:
      return_value =
            sys_ioctl((int)params[0],(int)params[1],(void *)params[2]);
      break;
    case SYS_LSEEK:
      return_value =
            sys_lseek((int)params[0],(int32)params[1],(int)params[2]);
      break;
    case SYS_FSTAT:
      return_value =
            sys_fstat ((int)params[0],(struct stat *)params[2]);
      break;
    case SYS_CLOSE:
      return_value = sys_close((int)params[0]);
      break;
    /* processes system calls */
    case SYS_FORK:
      return_value = sys_fork ();
      break;
    case SYS_EXIT:
      sys_exit ( (int)params[0]);
      return_value = 0;
      break;
    case SYS_GETPID:
      return_value = sys_getpid ();
      break;
    case SYS_GETPPID:
      return_value = sys_getppid ();
      break;
    case SYS_WAIT:
      return_value = sys_wait ( (int *)params[0]);
      break;
    case SYS_WAITPID:
      {
        uint32 status;
        return_value = sys_waitpid ((pid_t)params[0], &status,
                                    (int)params[2]);
        *((int *)params[1]) = status;
      }
      break;
    case SYS_USLEEP:
      sys_usleep ( (unsigned long)params[0] );
      return_value = 0;
      break;
    case SYS_SLEEP:
      sys_sleep ( (unsigned int)params[0] );
      return_value = 0;
      break;
    case SYS_SETPRIORITY:
      task_change_priority ( current_task, (unsigned int)params[0] );
      return_value = 0;
      break;
    /* thread system calls */
    case SYS_PTHREAD_CREATE:
      sys_pthread_create ( (pthread_t *)params[0], (pthread_attr_t *)params[1],
                          (void * (*)(void *)) params[2],
                          (void *) params [3] );
      return_value = 0;
      break;
    case SYS_PTHREAD_JOIN:
      return_value = sys_pthread_join ( (pthread_t *)params [0],
                                        (void **) params[1]);
      break;
    /* sched system calls */
    case SYS_SCHED_SETPARAM:
      {
        struct sched_param * p = (struct sched_param *)params[1];
        if (task_change_priority_by_tid ((int)params [0],p->sched_priority))
          return_value = 0;
        else
          return_value = -ESRCH;
      }
      break;
    default:
      kprintf("Invalid system call %d\r\n",syscall_no);
      return_value = -ENOSYS;
      break;
  }
  current_task->syscall.return_value = return_value;
}

void syscall_init(void)
{
  kprintf("Registering System Call Handler...");
	register_interrupt(SYSCALL,syscall);
  kprintf("OK\r\n");
}
