/*
 * sysdrv.c
 *-
 * Copyright (c) 2001, 2002, 2003 Corrado Santoro (csanto@diit.unict.it)
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Id: sysdrv.c,v 1.1.1.1 2005/05/15 09:50:01 corrado Exp $
 */

#include <kernel/ntypes.h>
#include <kernel/asm.h>
#include <kernel/kprintf.h>
#include <kernel/string.h>
#include <kernel/mem.h>
#include <kernel/mm.h>
#include <kernel/driver.h>
#include <kernel/queue.h>
#include <kernel/task.h>
#include <kernel/file.h>
#include <kernel/errno.h>
#include <kernel/profile.h>
#include <kernel/kernel.h>
#include <kernel/clist.h>
#include <user/stdio.h>

typedef struct {
  t_stream * stream;
  uint32  tid;
} t_sysinfo_thread;

void _sysdrv_task_info(t_clist * c)
{
  task_struct * t;
  char data[64], state;
  char s[] = " TID  PTID  State     PRI      CPUTime\n";
  clist_put(c,s,strlen(s));
  // FIXME! this walking should be atomic.... but YOU CANNOT __cli !!
  t = task_list.begin;
  do
  {
    state = '?';
    switch (t->state)
    {
      case TASK_RUNNING:  state = 'X'; break;
      case TASK_READY:    state = 'R'; break;
      case TASK_SLEEPING: state = 'S'; break;
      case TASK_STOPPED : state = 'T'; break;
      case TASK_ENDING:   state = 'Z'; break;
    }
    sprintf(data,"%4ld  %4ld     %c   %8ld  %6ld.%06lu\n",
            t->tid, t->p_tid, state, t->priority,
            t->cpu_time.tv_sec, t->cpu_time.tv_usec);
    clist_put (c, data, strlen(data));
    t = t->next_ordered;
  }
  while (t != task_list.begin);
  *data = 0;
  clist_put (c, data, 1);
}

void _sysdrv_mem_info (t_clist * c)
{
  char data[64];
  int32 kbusy,kfree;
  mem_stat(&kbusy,&kfree);
  sprintf(data,"Memory: Total %ld KB, Used %ld KB, Free %ld KB\n",
      (uint32)machine_memory_size / 1024l,kbusy / 1024l,kfree / 1024l);
  clist_put (c, data, strlen (data));
  *data = 0;
  clist_put (c, data, 1);
}

void _sysdrv_release_info (t_clist * c)
{
  char data[64];
  sprintf(data,"NUXI Release %s\n",KERNEL_RELEASE);
  clist_put(c, data, strlen(data));
  *data = 0;
  clist_put(c, data, 1);
}

int _sysdrv_open(file * f,char * name,int mode)
{
  t_clist * c;
  c = clist_new();
  if (c == NULL)
    return -ENOMEM;
  f->private_data = c;
  if (!strcmp(name,"tasks"))
    _sysdrv_task_info (c);
  else if (!strcmp(name,"mem"))
    _sysdrv_mem_info (c);
  else if (!strcmp(name,"release"))
    _sysdrv_release_info (c);
  else {
    clist_delete(c);
    return -ENOENT;
  }
  return 0;
}

int32 _sysdrv_read(file * f,char * buf,int32 len)
{
  t_clist * c;
  uint8  data;
  int count = 0;
  c = (t_clist *)f->private_data;
  while (len > 0)
  {
    data = clist_get(c);
    if (data == 0)
    {
      f->eof = 1;
      return count;
    }
    buf[count] = data;
    ++count;
    --len;
  }
  return count;
}

int32 _sysdrv_write(file * f,char * buf,int32 len)
{
  return -EPERM;
}

int _sysdrv_close(file * f)
{
  t_clist * c;
  c = (t_clist *)f->private_data;
  clist_delete(c);
  f->private_data = NULL;
  return 0;
}

t_file_drv_operations sysdrv_ops = {
  _sysdrv_open,
  _sysdrv_read,
  _sysdrv_write,
  _sysdrv_close,
  NULL,    // ioctl
  NULL,    // lseek
  NULL     // fstat
};

void sysdrv_init(void)
{
  if (!register_file_driver("sys",&sysdrv_ops))
  {
    kprintf("Error registering sysdrv driver");
    __cli();
    for (;;) ;
  }
  kprintf("Sysinfo ");
}
