/*
 * task.c
 *-
 * Copyright (c) 2001, 2002, 2003 Corrado Santoro (csanto@diit.unict.it)
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Id: task_old.c,v 1.1.1.1 2005/05/15 09:50:01 corrado Exp $
 */

#include <kernel/ntypes.h>
#include <kernel/io.h>
#include <kernel/asm.h>
#include <kernel/interrupt.h>
#include <kernel/kprintf.h>
#include <kernel/kernel.h>
#include <kernel/task.h>
#include <kernel/gdt.h>
#include <kernel/mm.h>
#include <kernel/mem.h>
#include <kernel/timer.h>
#include <kernel/profile.h>
#include <kernel/hz.h>
#include <kernel/signals.h>
#include <kernel/file.h>
#include <kernel/wait.h>
#include <user/signal.h>

t_profile_info profile_info;

//int pri = 10;

t_task_list task_list = { NULL,NULL,0,0};

task_struct * current_task = NULL;
task_struct * tasks[MAX_TASK_PRIORITY];

wait_chan wait_sys_call = NULL;

int scheduler_lock = 0;

char first_task_call = 1;

void task_call(uint32 tss)
{
  uint32 sel[2];
  unsigned long flags;
  sel[0] = 0;
  sel[1] = tss;

  if (first_task_call)
  {
    first_task_call = 0;
    // Dario Russo:
    // This is the first task, so use a workaround to call it
    // in order to allow NUXI to work with VMWare
    __save_flags_cli(flags);
    current_task->tss.link=tss;

    asm("pushfl;"
        "movw    %0,%%ax;"
        "ltr     %%ax;"
        "popfl;"
        "movw   %1,%%ax;"
        "movl   %2,%%edi;"
        "movw   %%ax,%%ss;"
        "movl   %%edi,%%esp;"
        "pushl  %3;"
        "pushl  %4;"
        "pushl  %5;"
        "movw   %6,%%ax;"
        "movw   %7,%%bx;"
        "movw   %%ax,%%ds;"
        "movw   %%bx,%%es;"
        "movw   $0x20,%%ax;"
        "out    %%al,$0x20;"
        : : "m" (tss),
            "m" (current_task->tss.ss),
            "m" (current_task->tss.esp),
            "m" (current_task->tss.eflags),
            "m" (current_task->tss.cs),
            "m" (current_task->tss.eip),
            "m" (current_task->tss.ds),
            "m" (current_task->tss.es) : "eax", "ebx", "esi", "edi");

    __restore_flags(flags);
    asm("iret");
  }
  else
    // enable 8259-1 timer interrupt and jump to new task
    asm("mov $0x20,%%ax;out %%al,$0x20;sti;ljmp %0;cli"::"m" (*sel) : "eax");
}

void task_prepare(task_struct * task,startup_proc proc,void * param)
{
  register uint32 * sp;

  kmemset((uint8 *)task, 0, sizeof (task_struct));
  task->stack = kmalloc(TASK_STACK_SIZE);
  sp = (uint32*)((uint32)task->stack + TASK_STACK_SIZE - 8);
  *sp = (uint32)param;
  sp--;
  task->tss.esp0 = (int32)sp;
  task->tss.ss0 = KERNEL_STACK_SELECTOR;
  task->tss.eax = 0;
  task->tss.ebx = 0;
  task->tss.ecx = 0;
  task->tss.edx = 0;
  task->tss.esi = 0;
  task->tss.edi = 0;
  task->tss.esp1 = task->tss.esp2 = task->tss.ss1 = task->tss.ss2 = 0;
  task->tss.cr3 = 0;
  task->tss.eip = (uint32)proc;
  task->tss.eflags = 0x0202; // 0x4202 -> paging on
  task->tss.esp = task->tss.esp0;
  task->tss.ss = task->tss.ss0;
  task->tss.cs = KERNEL_CODE_SELECTOR;
  task->tss.ds = task->tss.es =
  task->tss.fs = task->tss.gs = KERNEL_DATA_SELECTOR;
  task->tss.ldt = task->tss.debugtrap = task->tss.iomapbase = 0;
  task->priority = DEFAULT_PRIORITY;
}

// final task preparation and run
void task_make_in_list(uint32 * tid, task_struct * task)
{
  int pri = task->priority;
  // link new task in the ordered task list
  task->next_ordered = NULL;
  if (task_list.begin == NULL)
  {
    int i;
    // this is the first task
    for (i = 0;i < MAX_TASK_PRIORITY;i++) tasks[i] = NULL;
    task_list.begin = task;
    task_list.end = task;
  }
  else
  {
    task_list.end->next_ordered = task;
    task_list.end = task;
  }
  // link new task in the priority queues
  if (tasks[pri] == NULL)
  {
    tasks[pri] = task;
    task->next = task;
    task->prev = task;  // circular list, link with itself
  }
  else
  {
    task_struct * head;
    head = tasks[pri];     // head of the priority queue
    // insert new task between head->next and head->next->next
    task->next = head->next;
    task->prev = head->next->prev;
    head->next = task;
    task->next->prev = task;
  }
  // increment the total number
  ++task_list.total_tasks;
  ++task_list.last_tid;
  // set the tid and the parent tid
  task->tid = task_list.last_tid;
  if (current_task == NULL)
    task->p_tid = 0;
  else
    task->p_tid = current_task->tid;
  if (tid != NULL) *tid = task->tid;
  task->cpu_time.tv_sec = 0;
  task->cpu_time.tv_usec = 0;
  task->state = TASK_READY;
}

void task_run (task_struct * task)
{
  task_struct * saved_current = NULL;
  // Update current_task. The created task will be scheduler immediatelly
  if (current_task != NULL)
  {
    saved_current = current_task;
    saved_current->state = TASK_READY;
  }
  current_task = task;
  current_task->state = TASK_RUNNING;
  // Now map the KERNEL_TSS selector as a Task State Segment descriptor
  gdt_make_tss(KERNEL_TSS,(uint32)&task->tss,sizeof(task_struct),0);
  //display_gdt(KERNEL_TSS);
  // Now run the new task by performing a long jump using
  // the selector KERNEL_TSS
  task_call(KERNEL_TSS);
  current_task = saved_current;
  __sti();
}

// Start a new task!
void task_add(uint32 * tid,startup_proc proc,void * param)
{
  task_struct * task;
  // clear interrupt to prevent task switching
  __cli();
  // first allocate memory for task struct
  task = (task_struct *)kmalloc(sizeof(task_struct));
  if (task == NULL)
  {
    kprintf("No more space for tasks\r\n");
    __sti();
    return ;
  }
  // OK, first prepare the TSS in the task struct, using "proc" as
  // entry point
  task_prepare(task,proc,param);
  // now make the task run
  task_make_in_list(tid, task);
  task_run (task);
}

// fork current task
void task_fork_prepare(task_struct * parent, void * proc)
{
  task_struct * task;
  // clear interrupt to prevent task switching
  // first allocate memory for task struct
  task = (task_struct *)kmalloc(sizeof(task_struct));
  if (task == NULL)
  {
    kprintf("No more space for tasks\r\n");
    parent->fork_ret_val = -1;
    return ;
  }
  // OK, first prepare the TSS in the task struct
  task_prepare(task, proc, NULL);
  parent->forked_task = task;
}

// adjust the task structure to allow fork
void task_fork (task_struct * parent)
{
  task_struct * task;
  uint32 tid;
  register uint32 sp,sp0,bp;
  int i;

  task = parent->forked_task;
  // save stack pointer of new task
  sp = (uint32)task->stack;

  // copy the entire task structure
  kmemcpy((uint8 *)task, (uint8 *)parent, sizeof (task_struct));

  // increment the reference count of each open file
  for (i = 0; i < MAX_FILES;i++)
    if (current_task->files[i] != NULL)
      current_task->files[i]->ref_count++;

  // copy the stack
  task->stack = (uint8 *)sp;
  kmemcpy((uint8 *)task->stack, (uint8 *)parent->stack, TASK_STACK_SIZE);

  // calculate the ESP register for new task
  sp = parent->tss.esp - (uint32)parent->stack;
  sp0 = parent->tss.esp0 - (uint32)parent->stack;
  bp = parent->tss.ebp - (uint32)parent->stack;

  task->tss.esp =  (uint32)task->stack + sp;
  task->tss.esp0 = (uint32)task->stack + sp0;
  task->tss.ebp =  (uint32)task->stack + bp;
  task->tss.link = 0;

  // do not inherit pending signals
  task->signal_bit_vector = 0;
  task->signal_handled = 0;

  // prepare return values
  task->fork_ret_val = 0;
  // now make the task run
  task_make_in_list (&tid, task);
  parent->fork_ret_val = tid;
}

void task_change_priority (task_struct * task, int newprio)
{
  int old_priority;
  LOCK_SCHEDULER;
  if (task->priority != newprio)
  {
    old_priority = task->priority;
    // first remove task from current priority queue
    if (task->next == task->prev)
    {
      // this is the sole task in the priority queue
      tasks [old_priority] = NULL;
    }
    else
    {
      if (tasks[old_priority] == task)
      {
        // if the head of the priority queue points to this task
        // then switch to next task
        tasks[old_priority] = task->next;
      }
      // now remove task from queue
      task->prev->next = task->next;    // 'prev/next' linkage
      task->next->prev = task->prev;    // 'next/prev' linkage
    }
    // now insert task in the queue relevant to new priority
    task->priority = newprio;
    // link new task in the priority queues
    if (tasks[newprio] == NULL)
    {
      tasks[newprio] = task;
      task->next = task;
      task->prev = task;  // circular list, link with itself

    }
    else
    {
      task_struct * head;
      head = tasks[newprio];     // head of the priority queue
      // insert new task between head->next and head->next->next
      task->next = head->next;
      task->prev = head->next->prev;
      head->next = task;
      task->next->prev = task;
    }
  }
  UNLOCK_SCHEDULER;
}

// end current task
// FIXME! Manage children inheritance when the parent ends
void task_end (uint32 end_code)
{
  int i;
  task_struct * parent;
  __sti ();
  for (i = 0; i < MAX_FILES;i++)
    if (current_task->files[i] != NULL)
      sys_close (i);
  LOCK_SCHEDULER; //__cli ();
  current_task->state = TASK_ENDING;
  current_task->exit_code = end_code;
  parent = task_find (current_task->p_tid);
  if (parent != NULL)
  {
    TASK_SET_SIGNAL (parent, SIGCHLD);
  }
  UNLOCK_SCHEDULER;
  thread_wakeup_all (&wait_sys_call);
  for (;;) ;
  //scheduler ();//asm ( "int $0x20" ::); // do scheduler
}

// remove a finished task from task lists
void task_remove (task_struct * removed_task)
{
  if (removed_task->state == TASK_ENDING)
  {
    // task is marked to be terminated, remove it
    // first remove it from the ordered list
    task_struct *pprev, *p;
    LOCK_SCHEDULER;
    p = task_list.begin;
    pprev = NULL;
    // walk the ordered list to find the TID
    while (p != NULL)
    {
      if (p->tid == removed_task->tid)
      {
        // TID found
        if (p == task_list.begin)
        {
          // the element is the first
          task_list.begin = p->next_ordered;
        }
        else if (p == task_list.end)
        {
          // the element is the last
          task_list.end = pprev;
          if (pprev != NULL) pprev->next_ordered = NULL;
        }
        else
        {
          // the element is in the middle
          pprev->next_ordered = p->next_ordered;
        }
        task_list.total_tasks--;
        break;  // break the while
      }
      pprev = p;
      p = p->next_ordered;
    }
    // now remove the task from the priority list
    if (removed_task->next == removed_task)
    {
      // no more tasks in this priority list
      tasks[removed_task->priority] = NULL;
    }
    else
    {
      // find the position in the priority list
      p = tasks [removed_task->priority];
      while (p->next != removed_task)
      {
        p = p->next;
      }
      p->next = removed_task->next;
    }
    kfree(removed_task->stack);
    kfree(removed_task);
    UNLOCK_SCHEDULER;
  }
}

// find a task given the tid (you should use cli and sti to make it atomic!)
task_struct * task_find (int tid)
{
  task_struct * p;
  p = task_list.begin;
  while ( (p != NULL) && (p->tid != tid) )
    p = p->next_ordered;
  return p;
}

// find a task given the parent tid
task_struct * task_find_by_parent (int tid)
{
  task_struct * p;
  p = task_list.begin;
  while ( (p != NULL) && (p->p_tid != tid) )
    p = p->next_ordered;
  return p;
}

void task_dump(void)
{
  kprintf(" %x ",current_task->tss.eip);
}

// find the signal with the lowest number to be handled
int task_find_lowest_signal (task_struct * t)
{
  int i, mask = 1;

  uint32 sigv = t->signal_bit_vector;

  for (i = 0;i < MAX_SIGNAL;i++, mask <<= 1)
    if (sigv & mask) return (i + 1);

  kprintf ("PANIC: a signal should be handled but it was not found\r\n");
  __cli ();
  __halt ();
}

void sig_task_end(int sig)
{
  task_end (0);
}

void sig_stop(int sig)
{
  current_task->saved_state = current_task->state;
  current_task->state = TASK_STOPPED;
}


sighandler_t _default_signal_actions [] = {
  sig_task_end, // SIGHUP
  sig_task_end, // SIGINT
  sig_task_end, // SIGQUIT
  sig_task_end, // SIGILL
  SIG_IGN,      // SIGTRAP
  sig_task_end, // SIGABRT, SIGIOT
  sig_task_end, // SIGBUS
  sig_task_end, // SIGFPE
  sig_task_end, // SIGKILL
  SIG_IGN,      // SIGUSR1
  sig_task_end, // SIGSEGV
  SIG_IGN,      // SIGUSR2
  sig_task_end, // SIGPIPE
  SIG_IGN,      // SIGALRM
  sig_task_end, // SIGTERM
  sig_task_end, // SIGSTKFLT
  SIG_IGN,      // SIGCHLD
  SIG_IGN,      // SIGCONT, handled in scheduler ()
  sig_stop,     // SIGSTOP
  sig_stop,     // SIGTSTP
  SIG_IGN,      // SIGTTIN
  SIG_IGN,      // SIGTTOU
  SIG_IGN,      // SIGURG
  SIG_IGN,      // SIGXCPU
  SIG_IGN,      // SIGXFSZ
  SIG_IGN,      // SIGVTALRM
  SIG_IGN,      // SIGPROF
  SIG_IGN,      // SIGWINCH
  SIG_IGN,      // SIGIO
  SIG_IGN,      // SIGPWR
  sig_task_end, // SIGSYS
} ;

// handle the signal
void sig_proc (void)
{
  sighandler_t p;
  p = current_task->signal_proc_vector[current_task->signal_handled - 1];
  if (p == SIG_DFL)
    p = _default_signal_actions [current_task->signal_handled - 1];
  //kprintf ("Task %d, got signal %ld, %08lx\r\n",
  //         current_task->tid, current_task->signal_handled, p);
  if (p != SIG_IGN)
    p (current_task->signal_handled);
  current_task->signal_handled = -1;
  asm ("cli\n int $0x20"); // force the scheduler
  for (;;) ;
}

// prepare structures for signal handling
void prepare_signal_handling (task_struct * t, int sig)
{
  //kprintf ("Task %d, got signal %ld\r\n", t->tid, sig);
  t->saved_tss = t->tss;
  t->signal_handled = sig;
  t->tss.esp -= 8;
  t->tss.esp0 -= 8;
  t->tss.eip = (uint32)sig_proc;
}

void end_signal_handling (task_struct * t)
{
  current_task->signal_handled = 0;
  current_task->tss = current_task->saved_tss;
}

// This is the scheduler! It is very simple!
void scheduler(void)
{
  task_struct * prev,* end,* p;
  struct timeval temp;
  int i;
redo:
  if (current_task == NULL)
  {
    kprintf("PANIC! Scheduler is trying to run without tasks!\r\n");
    __cli();
    for (;;) ;
  }
  temp.tv_usec = do_timer();
  temp.tv_sec = 0;
  time_add(&current_task->cpu_time,&temp);

  // do not perform task switch if scheduler is locked
  if (scheduler_lock > 0)
  {
    gdt_make_tss(KERNEL_TSS,
                 (uint32)&current_task->tss,sizeof(task_struct),0);
    task_call(KERNEL_TSS);
    goto redo;
  }

  if (current_task->state == TASK_RUNNING) current_task->state = TASK_READY;
  // check if one or more timers are elapsed
  check_elapsed_timers();
  // first of all check a pending signal for all tasks
  p = task_list.begin;
  while (p != NULL)
  {
    if (p->fork_request)
    {
      p->fork_request = 0;
      task_fork (p);
      break;
    }
    else if ( (p->signal_bit_vector != 0) && (p->signal_handled == 0) )
    {
      if (p->state == TASK_SLEEPING)
        // a signal was sent to the task, please make it ready!
        p->state = TASK_READY;
      else if (p->state == TASK_STOPPED)
      {
        if (TASK_IS_SIG_SET(p, SIGCONT))
        {
          // a CONT signal was sent to the task, please restart it
          TASK_CLEAR_SIGNAL (p, SIGCONT);
          p->state = p->saved_state;
          if (p->state == TASK_RUNNING) p->state = TASK_READY;
        }
      }
    }
    p = p->next_ordered;
  }
  // OK! This is a simple round-robin with priorities
  for (i = 0;i < MAX_TASK_PRIORITY;i++)
  {
    current_task = tasks[i];
    if (current_task != NULL)
    {
      end  = current_task;
      prev = NULL;
      do
      {
        prev = current_task;
        current_task = current_task->next;
        if (current_task->state == TASK_READY)
        {
          // We found the task to run
          tasks[i] = current_task;
          if ( (current_task->signal_bit_vector != 0)
            && (current_task->signal_handled == 0) )
          {
            // a signal was caugth, handle it!
            int sig;
            sig = task_find_lowest_signal (current_task);
            //kprintf ("Task %d, got signal %d\r\n",current_task->tid,sig);
            TASK_CLEAR_SIGNAL (current_task, sig);
            prepare_signal_handling (current_task, sig);
          }
          if (current_task->signal_handled == -1)
            end_signal_handling (current_task);
          // Since we use always the same selector to run the current_task,
          // let's update the KERNEL_TSS selector in the GDT with the TSS
          // of the new task to be scheduled
          gdt_make_tss(KERNEL_TSS,
                      (uint32)&current_task->tss,sizeof(task_struct),0);
          current_task->state = TASK_RUNNING;
          // well, switch to the new task
          task_call(KERNEL_TSS);
          // then restart scheduler
          goto redo;
        }
      }
      while (current_task != end);
      // we walked the entire priority list without finding a ready task
    }
  }
  kprintf("PANIC! Scheduler did not found a ready tasks. Cannot continue!\r\n");
  __halt();
}
