/*
 * unistd.c
 *-
 * Copyright (c) 2001, 2002, 2003 Corrado Santoro (csanto@diit.unict.it)
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Id: unistd.c,v 1.1.1.1 2005/05/15 09:50:01 corrado Exp $
 */

#include <kernel/ntypes.h>
#include <kernel/errno.h>
#include <kernel/syscall.h>
#include <user/sys/time.h>
#include <user/sys/resource.h>

int sleep(unsigned int secs)
{
  PREPARE_CALL(SYS_SLEEP);
  PREPARE_PARAM(0,(uint32)secs);
  MAKE_CALL;
  return SYS_RETVAL;
}

void usleep(unsigned long usecs)
{
  PREPARE_CALL(SYS_USLEEP);
  PREPARE_PARAM(0,(uint32)usecs);
  MAKE_CALL;
}

int setpriority (int which, int who, int prio)
{
  if ((which != PRIO_PROCESS) || (who != 0))
  {
    errno = EINVAL;
    return -1;
  }
  PREPARE_CALL(SYS_SETPRIORITY);
  PREPARE_PARAM(0,(uint32)prio + DEFAULT_PRIORITY);
  MAKE_CALL;
  return 0;
}

