/*
 * wait.c
 *-
 * Copyright (c) 2001, 2002, 2003 Corrado Santoro (csanto@diit.unict.it)
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Id: wait.c,v 1.1.1.1 2005/05/15 09:50:01 corrado Exp $
 */

#include <kernel/wait.h>
#include <kernel/asm.h>
#include <kernel/task.h>
#include <kernel/timer.h>
#include <kernel/mm.h>
#include <kernel/kprintf.h>

void in_int_sleep (wait_chan * wchan, sw_timer * timer_data)
{
  wait_list * p;
  p = (wait_list *)kmalloc (sizeof (wait_list));
  if (p == NULL) {
    kprintf("No more memory for wait list allocation\r\n");
    return ;
  }
  p->task = current_task;
  p->timer_data = timer_data;
  p->skip = 0;
  p->next = *wchan;
  if (timer_data != NULL)
    timer_data->wait_chan_data = p;
  current_task->state = TASK_SLEEPING;
  *wchan = p;
  //__cli ();
  asm ( "int $0x20" ::); // do scheduler
}

void thread_sleep (wait_chan * wchan)
{
  unsigned long flags;
  __save_flags_cli (flags);
  in_int_sleep (wchan, NULL);
  __restore_flags (flags);
}

// FIXME It is buggy... hangs up scheduler at the exit!!!!
// wait until condition or timeout (timeout is in microsecs)
// returns 1 if timer elapsed, 0 if a thread_wakeup is issued
int thread_timed_sleep (wait_chan * wchan, unsigned long timeout)
{
  sw_timer  *aux;
  unsigned long flags;
  int return_value;

  aux = timer_create (timeout);
  if (aux == NULL)
    return -1;
  __save_flags_cli (flags);
  //LOCK_SCHEDULER ;
  // insert timer
  timer_insert (aux);
  //UNLOCK_SCHEDULER ;
  // goto sleeping...
  in_int_sleep (wchan, aux);
  return_value = (*wchan != NULL);
  __restore_flags (flags);
  return return_value;
}

void thread_sleep_and_start_scheduler (wait_chan * wchan)
{
  wait_list * p;
  unsigned long flags;
  p = (wait_list *)kmalloc (sizeof (wait_list));
  if (p == NULL) {
    kprintf("No more memory for wait list allocation\r\n");
    return ;
  }
  __save_flags_cli (flags);
  p->task = current_task;
  p->next = *wchan;
  p->timer_data = NULL;
  p->skip = 0;
  *wchan = p;
  current_task->state = TASK_SLEEPING;
  UNLOCK_SCHEDULER;

  asm ( "int $0x20" ::); // do scheduler
  __restore_flags (flags);
}

void in_int_wakeup (wait_chan * wchan)
{
  wait_list * p;
  sw_timer * td;
  if (*wchan != NULL)
  {
    if (!(*wchan)->skip)
    {
      td = (*wchan)->timer_data;
      if (td != NULL)
        td->skip = 1;
      (*wchan)->task->state = TASK_READY;
    }
    p = *wchan;
    *wchan = p->next;
    kfree (p);
  }
}

void in_int_wakeup_all (wait_chan * wchan)
{
  while (*wchan != NULL)
  {
    in_int_wakeup (wchan);
  }
}

void thread_wakeup (wait_chan * wchan)
{
  unsigned long flags;
  __save_flags_cli (flags);
  in_int_wakeup (wchan);
  __restore_flags (flags);
}

void thread_wakeup_all (wait_chan * wchan)
{
  unsigned long flags;
  __save_flags_cli (flags);
  in_int_wakeup_all (wchan);
  __restore_flags (flags);
}

void semaphore_up (t_semaphore * sem)
{
  unsigned long flags;
  __save_flags_cli (flags);
  ++sem->value;
  in_int_wakeup (&sem->wchan);
  __restore_flags (flags);
}

void semaphore_down (t_semaphore * sem)
{
  unsigned long flags;
  __save_flags_cli (flags);
  while (sem->value == 0)
    in_int_sleep (&sem->wchan, NULL);
  sem->value--;
  __restore_flags (flags);
}


void cond_wait (t_cond * cond, t_semaphore * sem)
{
  unsigned long flags;
  __save_flags_cli (flags);
  ++sem->value;
  cond->sem = sem;
  in_int_wakeup (&sem->wchan);
  in_int_sleep (&cond->wchan, NULL);
  while (sem->value == 0)
    in_int_sleep (&sem->wchan, NULL);
  sem->value--;
  __restore_flags (flags);
}

void cond_signal (t_cond * cond)
{
  unsigned long flags;
  __save_flags_cli (flags);
  if (cond->wchan != NULL)
  {
    // someone is waiting
    in_int_wakeup (&cond->wchan);
  }
  __restore_flags (flags);
}
