#
# NUXI Boot Code
# Copyright (C) 2001 Corrado Santoro <csanto@diit.unict.it>
#
# This code is loaded from BIOS at address 0000:7C00
#

.include "segments.s"

.global boot

	.code16
	.balign	0x08
boot:
  jmp next
sector_count: .word 0
sector:	.byte	2
head:	.byte	0
track:	.byte	0
address: .word	kaddress

force_size:     .word   0       # Use this size instead of the one in BIOS vars

# Write to indexed VGA register (AL=index, AH=data, DX=index reg. port)
outidx: outb    %al, %dx
        pushw   %ax
        movb    %ah, %al
        incw    %dx
        outb    %al, %dx
        decw    %dx
        popw    %ax
        ret
							
next:
#ifndef	EMBEDDED  # this is a remark???
	
        movw    $0x3cc, %dx                     # Get CRTC port
        inb     %dx, %al
        movb    $0xd4, %dl
        rorb    %al                             # Mono or color?
        jc      set48a
        movb    $0xb4, %dl
set48a: movw    $0x0c11, %ax            # Vertical sync end (also unlocks CR0-7)
        call    outidx
        movw    $0x0b06, %ax                    # Vertical total
        call    outidx
        movw    $0x3e07, %ax                    # (Vertical) overflow
        call    outidx
        movw    $0xea10, %ax                    # Vertical sync start
        call    outidx
        movw    $0xdf12, %ax                    # Vertical display end
        call    outidx
        movw    $0xe715, %ax                    # Vertical blank start
        call    outidx
        movw    $0x0416, %ax                    # Vertical blank end
        call    outidx
        pushw   %dx
        movb    $0xcc, %dl                      # Misc output register (read)
        inb     %dx, %al
        movb    $0xc2, %dl                      # (write)
        andb    $0x0d, %al      # Preserve clock select bits and color bit
        orb     $0xe2, %al                      # Set correct sync polarity
        outb    %al, %dx
        popw    %dx
        movw    $0x501e, force_size
        stc                                     # That's all.
																						

	movw    $0x1112, %ax                    # Use 8x8 font
        xorb    %bl, %bl
        int     $0x10
        movw    $0x1200, %ax                    # Use alternate print screen
        movb    $0x20, %bl
        int     $0x10
        movw    $0x1201, %ax                    # Turn off cursor emulation
        movb    $0x34, %bl
        int     $0x10
        movb    $0x01, %ah                      # Define cursor scan lines 6-7
        movw    $0x0607, %cx
        int     $0x10
        stc
	movw    $0xdf12, %ax                    # VGA vertical display end
	movw    $0x503c, force_size
	
	jmp	stepover
failure:
	call writedot
	jmp	failure
stepover:
#endif	
	


	call	writedot          # put a dot onto the screen to signal
                          # the first boot stage
	movw	$0x7c0,%ax        # boot sector is loaded at 07C0H:0000H
	movw	%ax,%ds           # thus load DS to access this area

	movw	sector_count,%cx  # loop for the number of sectors
                          # to read
do_read:
	pushw	%cx
	movw	$ksegment, %ax    # prepare register for sector read
	movw	%ax, %es          # address (segment)
	movw	address, %bx      # address (offset)
	movb	$0x02, %ah        # service 2 (read sectors)
	movb	$0x01,%al         # sector count=1 sector
	movb	sector, %cl       # sector number
	movb	track,%ch         # track number
	movb	head,%dh          # head number
	movb	$0x0,%dl          # drive number (0=A:)
	int	$0x13               # read a sector of microkernel image
	jnc	read_ok

	movb	$0x45,%al          # if error, display 'E' and retry
	call	writechar
	popw	%cx
	jmp	do_read

read_ok:
	call	writedot           # sector loaded successfully
                           # now go to next sector
	addw	$512,address       # sector size if 512 bytes
	incb	sector
	cmpb	$19,sector         # are we at the end of track?
                           # (1.44 MB floppy => 18 secs./track)
	jne	read_next
	movb	$1,sector
	incb	head               # end of track reached => next disk face
	cmpb	$2,head            # did we read both faces ?
	jne	read_next
	movb	$0,head            # both faces read, go to next track
	incb	track
read_next:
	popw	%cx
	loop	do_read           # loop to total number of sector
                          # to read

	movw	$0x03F2, %dx      # turn off drive motor
	xorb	%al, %al
	outb	%al, %dx
  
	call	writedot           # motor off successfully

	ljmp	$ksegment, $kaddress
  
  
# write a dot into the screen
writedot:
	movb	$0x2e,%al
writechar:
	movb	$0x0e,%ah
	movw	$0x0007,%bx
	int	$0x10
	ret

	.org	510, 0
  .byte 0x55,0xAA

