/*
 * kernel.c
 * Copyright (C) 2001, 2002 Corrado Santoro (csanto@diit.unict.it)
 */

#include <kernel/io.h>
#include <kernel/asm.h>
#include <kernel/console.h>
#include <kernel/interrupt.h>
#include <kernel/timer.h>
#include <kernel/task.h>
#include <kernel/kprintf.h>
#include <kernel/mm.h>
#include <kernel/wait.h>
#include <kernel/exception.h>
#include <kernel/mailbox.h>
#include <kernel/driver.h>
#include <user/pthread.h>

#define KERNEL_RELEASE  "0.0.6-2"

void * machine_memory_size = 0;

void memory_test(void)
{
  // Start memtest from 0x100000 (1 MB)
  // Declare as 'volatile' otherwise assignment and comparison with 0x1234
  // does not work due to compiler optimization
  volatile uint16 * memaddr = (uint16 *)0x100000;
  while (memaddr != 0x0) {
    uint32 l_addr = (uint32)memaddr;
    l_addr >>= 20;
    kprintf("\rTesting Memory: %d MBytes",(int)l_addr);
    *memaddr = 0x1234;
    if (*memaddr != 0x1234)
      break;
    memaddr += 0x80000; // goto next megabyte
  }
  kprintf(" Found OK\r\n");
  machine_memory_size = (void *)memaddr;
}

void a20_enable(void)
{
  kprintf ("Enabling A20...");
  outb(0xd1,0x64);
  __SLOW_DOWN_IO;
  outb(0xdf,0x60);
  __SLOW_DOWN_IO;
  outb(inb(0x92) | 0x02,0x92); // fast "A20" version
  __SLOW_DOWN_IO;
  __SLOW_DOWN_IO;
  __SLOW_DOWN_IO;
  __SLOW_DOWN_IO;
  kprintf ("OK\r\n");
}

void reset_coprocessor(void)
{
  kprintf ("Resetting coprocessor...");
  outb(0xf0,0x0);
  __SLOW_DOWN_IO;
  outb(0xf1,0x0);
  __SLOW_DOWN_IO;

    
  __asm__("finit
	  clts
	  mov %cr0,%eax
	  and $0xfffffff1,%eax
	  mov %eax,%cr0
	  ");
  
  kprintf ("OK\r\n");
}

void * fourth_task(void *);
void * third_task(void * x);
void * second_task(void * x);
void * producer(void * x);
void * consumer(void * x);
void * shell(void * x);

extern wait_chan w1;

void main_kernel_task(void * x)
{
  unsigned char c;
  pthread_t t1,t2,t3;

  kprintf("Main Task Started OK\r\n");
  init_timer();
  kprintf("Starting User Tasks...\r\n");
  //print_mem_space();
  pthread_create(&t1,NULL,shell,(void *)NULL);
  for (;;);
  pthread_create(&t1,NULL,consumer,(void *)NULL);
  pthread_create(&t2,NULL,producer,(void *)NULL);
  for (;;);
  pthread_create(&t1,NULL,second_task,(void *)"Second");
  pthread_create(&t2,NULL,third_task,(void *)"Third");
  pthread_create(&t3,NULL,fourth_task,(void *)"Fourth");
  for (;;)
  {
  c++;
  if(c>=4) c=0;
  spinner(c,1);
/*  c = 0x20;
  for (;;) {
    console_memput(0,10,c,7);
    //kprintf("First %d ",c);
    c++;
    if (c >= 0x80) {
      c = ' ';
      //wakeup(&w1);
    }
*/  }
}

void kernel_main()
{
  console_base_init();
  kprintf ("\r\nNUXI release " KERNEL_RELEASE " loaded.\r\n");
  kprintf ("Copyright (C) 2001,2002 Corrado Santoro (csanto@diit.unict.it)" \
      " GPL Released.\r\n");
  kprintf ("Compiled on " __DATE__ "\r\n");
  kprintf ("Booting the kernel.\r\n");
  a20_enable();
  memory_test();
  reset_coprocessor();
  interrupt_setup();
  kprintf("Preparing Memory Manager...");
  // hummm.. we consider heap memory starting from second megabyte
  // indeed we should start from the end of kernel (how to compute it?)
  mm_init(0x100000,(uint32)machine_memory_size);
  kprintf("OK\r\n");
  driver_init();
  mailbox_init();
  kprintf("Preparing Task Structures...\r\n");
  task_add(NULL,main_kernel_task,NULL);
  // never return
  __cli();
  kprintf("PANIC! Unreachable point reached!\r\n");
  for (;;) ;
}
