/*
 * mailbox.c
 * Copyright (C) 2001,2002 Corrado Santoro (csanto@diit.unict.it)
 */

#include <kernel/mailbox.h>
#include <kernel/mm.h>
#include <kernel/mem.h>
#include <kernel/kprintf.h>

t_mailbox_entry mailboxes[MAX_MAILBOX];

t_semaphore mb_mutex = INIT_NEW_SEMAPHORE(1);
t_cond mb_cond = INIT_NEW_COND;


// FIXME! A lot of race conditions in the case of mailbox destruction!

void mailbox_init(void)
{
  int i;
  kprintf("Initializing Mailboxes...");
  for (i = 0;i < MAX_MAILBOX;i++)
  {
    NEW_SEMAPHORE(mailboxes[i].connections,0);
    mailboxes[i].mb = NULL;
    mailboxes[i].in_use = 0;
  }
  kprintf(" %d mailboxes OK\r\n",MAX_MAILBOX);
}

int mailbox_create(int mailbox_num)
{
  int retval = 0;
  mailbox_num--;
  semaphore_down(&mb_mutex);
  if (mailbox_num == -1)
  {
    do mailbox_num++;
    while ((mailboxes[mailbox_num].mb != NULL) && (mailbox_num < MAX_MAILBOX));
    if (mailbox_num >= MAX_MAILBOX)
    {
      retval = -1;
      goto do_exit;
    }
  }
  else
    if (mailboxes[mailbox_num].mb != NULL)
    {
      retval = -1;
      goto do_exit;
    }
  mailboxes[mailbox_num].mb = (t_mailbox *)kmalloc(sizeof(t_mailbox));
  if (mailboxes[mailbox_num].mb == NULL)
    retval = -1;
  else
  {
    semaphore_up(&mailboxes[mailbox_num].connections);
    NEW_SEMAPHORE(mailboxes[mailbox_num].mb->mutex,1);
    NEW_SEMAPHORE(mailboxes[mailbox_num].mb->empty,1);
    NEW_SEMAPHORE(mailboxes[mailbox_num].mb->full,0);
    mailboxes[mailbox_num].async_proc = NULL;
  }
do_exit:
  semaphore_up(&mb_mutex);
  return retval;
}

int mailbox_connect(int mailbox_num)
{
  t_mailbox_entry * mbe;
  mailbox_num--;
  semaphore_down(&mb_mutex);
  mbe = &mailboxes[mailbox_num];
  if (mbe->mb == NULL)
  {
    semaphore_up(&mb_mutex);
    // FIXME! Race conditions????
    semaphore_down(&mbe->connections);
  }
  else
    semaphore_up(&mb_mutex);
  return 0;
}

int mailbox_destroy(int mailbox_num)
{
  return -1;
}

int mailbox_set_async(int mailbox_num,mailbox_async_proc proc)
{
  t_mailbox_entry * mbe;
  mailbox_num--;
  mbe = &mailboxes[mailbox_num];
  if (mbe->mb == NULL)
  {
    return -1;
  }
  mbe->async_proc = proc;
  return 0;
}

int mailbox_send(int mailbox_num,t_mailbox_message * msg)
{
  t_mailbox_entry * mbe;
  mailbox_num--;
  mbe = &mailboxes[mailbox_num];
  if (mbe->mb == NULL)
  {
    return -1;
  }
  if (mbe->async_proc != NULL)
  {
    mbe->async_proc(mailbox_num,msg);
  }
  else
  {
    semaphore_down(&mbe->mb->empty);
    semaphore_down(&mbe->mb->mutex);
    kmemcpy((uint8 *)&mbe->mb->msg,(uint8 *)msg,sizeof(t_mailbox_message));
    semaphore_up(&mbe->mb->full);
    semaphore_up(&mbe->mb->mutex);
  }
  return 0;
}

int mailbox_receive(int mailbox_num,t_mailbox_message * msg)
{
  t_mailbox_entry * mbe;
  mailbox_num--;
  mbe = &mailboxes[mailbox_num];
  if (mbe->mb == NULL)
  {
    return -1;
  }
  semaphore_down(&mbe->mb->full);
  semaphore_down(&mbe->mb->mutex);
  kmemcpy((uint8 *)msg,(uint8 *)&mbe->mb->msg,sizeof(t_mailbox_message));
                                    //get item
  semaphore_up(&mbe->mb->mutex);
  semaphore_up(&mbe->mb->empty);
  return 0;
}
