/*
 * mailbox.h
 * Copyright (C) 2001,2002 Corrado Santoro (csanto@diit.unict.it)
 */

#ifndef __MAILBOX_H
#define __MAILBOX_H

#include <kernel/ntypes.h>
#include <kernel/task.h>
#include <kernel/wait.h>

#define MAX_MSG_PARAM 20

typedef struct {
  int type;
  uint32 parameter[MAX_MSG_PARAM];
} t_mailbox_message;

typedef struct {
  t_semaphore mutex;
  t_semaphore empty,full;
  t_mailbox_message msg;
} t_mailbox;

typedef void  (*mailbox_async_proc)(int,t_mailbox_message *);

typedef struct {
  t_semaphore connections;
  t_mailbox * mb;
  uint8 in_use;
  mailbox_async_proc async_proc;
} t_mailbox_entry;

#define MAX_MAILBOX  256

void mailbox_init(void);
int mailbox_create(int mailbox_num);
int mailbox_connect(int mailbox_num);
int mailbox_set_async(int mailbox_num,mailbox_async_proc proc);
int mailbox_send(int mailbox_num,t_mailbox_message * msg);
int mailbox_receive(int mailbox_num,t_mailbox_message * msg);
int mailbox_destroy(int mailbox_num);

#endif
