/*
 * rpc.c
 * Copyright (C) 2001,2002 Corrado Santoro (csanto@diit.unict.it)
 */

#include <kernel/rpc.h>
#include <kernel/mm.h>
#include <kernel/mem.h>
#include <kernel/task.h>
#include <kernel/mailbox.h>

t_rpc_svr_list * svr_list = NULL;
t_semaphore rpc_mutex = INIT_NEW_SEMAPHORE(1);

t_rpc_server * rpc_create_server(char * name,int svr_id,int maxproc)
{
  t_rpc_svr_list * p;
  t_rpc_server * svr = NULL;
  semaphore_down(&rpc_mutex);
  p = (t_rpc_svr_list *)kmalloc(sizeof(t_rpc_svr_list));
  if (p == NULL) goto do_exit;
  p->server.procs = (t_rpc*)kmalloc(sizeof(t_rpc)*maxproc);
  if (p->server.procs == NULL)
  {
    kfree(p);
    goto do_exit;
  }
  p->next = svr_list;
  svr_list = p;
  svr = &p->server;
  svr->nprocs = maxproc;
  svr->name = name;
  svr->svr_id = svr_id;
  NEW_SEMAPHORE(svr->mutex,1);
do_exit:
  semaphore_up(&rpc_mutex);
  return svr;
}

int rpc_register_call(t_rpc_server * svr,int id,char * procname,rpc_proc proc)
{
  svr->procs[id].name = procname;
  svr->procs[id].proc = proc;
  return 0;
}

typedef struct {
  t_mailbox_message * message;
  t_rpc_server * server;
} t_rpc_call_info;

void * serve_rpc(void * x)
{
  t_rpc_server * svr;
  t_mailbox_message msg;
  svr = ((t_rpc_call_info *)x)->server;
  kmemcpy((uint8 *)&msg,(uint8*)&((t_rpc_call_info *)x)->message,
                                sizeof(t_mailbox_message));
  semaphore_up(&svr->mutex);
  return NULL;
}

int rpc_start_server(t_rpc_server * svr)
{
  mailbox_create(svr->svr_id);
  for (;;)
  {
    t_mailbox_message msg;
    t_rpc_call_info ci;
    semaphore_down(&svr->mutex);
    mailbox_receive(svr->svr_id,&msg);
    ci.server = svr;
    ci.message = &msg;
    task_add(&svr->task_id,(startup_proc)serve_rpc,(void *)&ci);
  }
  return 0;
}
