/*
 * filesystem.h
 * Copyright (C) 2001,2002 Corrado Santoro (csanto@diit.unict.it)
 *
 *   The contents of this file are subject to the GNU Public License
 *   (the "License"); you may not use this file except in compliance with
 *   the License. You may obtain a copy of the License at http://www.fsf.org.
 *
 *   Software distributed under the License is distributed on an "AS IS"
 *   basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 *   License for the specific language governing rights and limitations
 *   under the License.
 *
 *   The Original Code is NUXI code.
 *
 *   The Initial Developer of the Original Code is Corrado Santoro.
 *   Portions created by Corrado Santoro are Copyright (C) 2001,2002.
 *   All Rights Reserved.
 */

#ifndef __FILESYSTEM_H
#define __FILESYSTEM_H

#include <kernel/driver.h>
#include <kernel/task.h>
#include <kernel/wait.h>
#include <kernel/dir.h>

#define FS_FILE_REGULAR   0
#define FS_FILE_DIRECTORY 1

#define FILESYSTEM_DRIVER_NAME  "#FS"
#define PATH_SEPARATOR          ':'

typedef struct
{
  char fs_name[MAX_FILENAME+1];
  char driver_name[MAX_FILENAME+1];
  char fstype_name[MAX_FILENAME+1];
  int  rpc_handle;
} t_mount_entry;

typedef struct
{
  int   type;
  char  name[MAX_FILENAME+1];
  long  size;
  void * filesystem;
  t_mount_entry * mount_info;
  void * fs_data;
} t_file_entry;

typedef void            (*t_fs_create_fs) (t_mount_entry *);
typedef t_file_entry *  (*t_fs_get_root) (t_mount_entry *);
typedef t_file_entry *  (*t_fs_get_entry) (t_mount_entry *,
                                           t_file_entry *, char * name);
typedef t_file_entry *  (*t_fs_scan_dir) (t_mount_entry *,
                                           t_file_entry *, DIR * stream);
typedef t_file_entry *  (*t_fs_create_entry) (t_mount_entry *,
                                              t_file_entry *,
                                              char * name,int type);
typedef int             (*t_fs_get_next_block) (t_mount_entry *,
                                           t_file_entry *, char * name,
                                           long file_position);

typedef struct
{
  t_fs_create_fs      fs_create_fs;
  t_fs_get_root       fs_get_root;
  t_fs_get_entry      fs_get_entry;
  t_fs_create_entry   fs_create_entry;
  t_fs_get_next_block fs_get_next_block;
} t_filesystem_operations;

int register_filesystem (char * name, t_filesystem_operations * p);
int fs_mount (char * fsname, char * driver, char * fstype);
t_mount_entry * mount_find (char * fs_name);
void fs_init(void);
int fs_open_dir(char * driver_name, char * pathname, DIR * dirent);


#endif
