/*
 * sysfs.c
 * Copyright (C) 2001,2002 Corrado Santoro (csanto@diit.unict.it)
 *
 *   The contents of this file are subject to the GNU Public License
 *   (the "License"); you may not use this file except in compliance with
 *   the License. You may obtain a copy of the License at http://www.fsf.org.
 *
 *   Software distributed under the License is distributed on an "AS IS"
 *   basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 *   License for the specific language governing rights and limitations
 *   under the License.
 *
 *   The Original Code is NUXI code.
 *
 *   The Initial Developer of the Original Code is Corrado Santoro.
 *   Portions created by Corrado Santoro are Copyright (C) 2001,2002.
 *   All Rights Reserved.
 */

#include <kernel/driver.h>
#include <kernel/task.h>
#include <kernel/wait.h>
#include <kernel/mem.h>
#include <kernel/mm.h>
#include <kernel/string.h>
#include <kernel/errno.h>
#include <kernel/kprintf.h>
#include <kernel/asm.h>
#include <kernel/filesystem.h>
#include <kernel/sysfs.h>
#include <kernel/hash.h>
#include <kernel/kernel.h>
#include <user/stdio.h>

t_file_entry sysfs_root;
t_file_entry * procdir, * memdir;


void _sysfs_create_fs (t_mount_entry * mount_entry)
{
  sysfs_root.type = FS_FILE_DIRECTORY;
  sysfs_root.fs_data = new_hash (HASH_TYPE_STRING);
  strcpy (sysfs_root.name, "/");
}


t_file_entry * _sysfs_get_root (t_mount_entry * mount_entry)
{
  return &sysfs_root;
}


t_file_entry * _sysfs_get_entry (t_mount_entry * mount_entry,
                                 t_file_entry * e, char * name)
{
  t_file_entry * he;
  he = find_hash (e->fs_data, name);
  return (t_file_entry *)he;
}


int _sysfs_get_next_block (t_mount_entry * mount_entry, 
                           t_file_entry * e, char * buf,
                           long file_offset)
{
  if (e == memdir)
  {
    int32 kbusy,kfree;
    mem_stat(&kbusy,&kfree);
    sprintf(buf,
      "Memory: Total %ld KB, Used %ld KB, Free %ld KB\n",
      (uint32)machine_memory_size / 1024l,kbusy / 1024l,kfree / 1024l);
    memdir->size = strlen (buf) + 1;
  }
  return 1;
}


t_file_entry * _sysfs_create_entry (t_mount_entry * mount_entry,
                                    t_file_entry * e,
                                    char * name, int type)
{
  t_file_entry * newfile;
  newfile = (t_file_entry *)kmalloc (sizeof (t_file_entry));
  if (newfile == NULL) return NULL;
  newfile->type = type;
  newfile->fs_data = NULL;
  newfile->size = 0;
  strcpy (newfile->name, name);
  add_hash (e->fs_data, name, newfile);
  return newfile;
}


t_filesystem_operations sysfs_ops = {
  _sysfs_create_fs,
  _sysfs_get_root,
  _sysfs_get_entry,
  _sysfs_create_entry,
  _sysfs_get_next_block,
};

void sys_fs_init(void)
{
  if (!register_filesystem ("sysfs", &sysfs_ops))
  {
    kprintf("Error registering sysfs");
    __cli();
    for (;;) ;
  }

  _sysfs_create_fs (NULL);
  procdir = _sysfs_create_entry (NULL, &sysfs_root, "proc", FS_FILE_DIRECTORY);
  memdir = _sysfs_create_entry (NULL, &sysfs_root, "mem",  FS_FILE_REGULAR);
  _sysfs_create_entry (NULL, &sysfs_root, "ipc",  FS_FILE_DIRECTORY);

  fs_mount ("sys", "null", "sysfs");
  kprintf("NUXI SYS-FS filesystem manager installed.\r\n");
}
