/*
 * userproc.c
 * Copyright (C) 2001, 2002 Corrado Santoro (csanto@diit.unict.it)
 *
 *   The contents of this file are subject to the GNU Public License
 *   (the "License"); you may not use this file except in compliance with
 *   the License. You may obtain a copy of the License at http://www.fsf.org.
 *
 *   Software distributed under the License is distributed on an "AS IS"
 *   basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 *   License for the specific language governing rights and limitations
 *   under the License.
 *
 *   The Original Code is NUXI code.
 *
 *   The Initial Developer of the Original Code is Corrado Santoro.
 *   Portions created by Corrado Santoro are Copyright (C) 2001,2002.
 *   All Rights Reserved.
 */

#include <kernel/hz.h>
#include <kernel/i8253.h>
#include <kernel/mm.h>
#include <kernel/rpc.h>

#include <pthread.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <fcntl.h>
#include <sys/time.h>
#include <unistd.h>
#include <errno.h>
#include <signal.h>
#include <sys/ioctl.h>
#include <sys/resource.h>

void spinner(int progress,int task)
{
  uint16 *vgamem=(uint16*)0xb8000;
	switch ( progress)
	{
		case 0:
      *vgamem = '-' + 0x700;
      break;
		case 1:
      *vgamem = '/' + 0x700;
      break;
		case 2:
      *vgamem = '|' + 0x700;
      break;
		case 3:
      *vgamem = '\\' + 0x700;
      break;
	}
			
}

void * second_task(void * x)
{
  unsigned char c=0;
  for (;;) {
	  spinner(c,2);
    c++;
    if (c >= 4) {
      c = 0;
    }
  }
}

void cat(char * name)
{
  int f,rd;
  char  c;
  f = open(name,O_RDONLY);
  if (f < 0)
  {
    printf("Error in open:%d\n",errno);return;
  }
  while ((rd=read(f,&c,1)) > 0)
  {
    putchar(c);
  }
  close(f);
}

void * bfifo_cons(void * x)
{
  int f,retval,i;
  char c[64];
  f = open("bfifo:0",O_CREAT | O_RDWR);
  if (f < 0)
  {
    printf("Error in open\n");
    return NULL;
  }
  //printf("!");
  for (;;)
  {
    retval = read(f,c,32);
    //printf("C%d",retval);
    if (retval < 0)
    {
      printf("Error in read\n");
      close(f);
      return NULL;
    }
    for (i = 0;i < retval;i++)
      putchar(c[i]);
    putchar('\n');
  }
  return NULL;
}

void bfifo_prod(void)
{
  int f;
  int n = 0;
  char c = ' ';
  char buf[32];
  printf("Testing Block FIFOs\n");
  f = open("bfifo:0",O_WRONLY);
  if (f < 0)
  {
    printf("Error in create\n");
    return;
  }
  for (n = 0;n < 200;n++)
  {
    memset(buf,32,c);
    if (write(f,buf,32) < 0)
    {
      printf("error in write\n");
      close(f);
      return ;
    }
    c++;
    if (c == 127) c = ' ';
  }
  close(f);
}

void fifo_prod(void)
{
  int f;
  int n = 0;
  char c = ' ';
  printf("Testing Character FIFOs\n");
  //setpriority (PRIO_PROCESS, 0, -1);
  f = open("cfifo:0",O_WRONLY);
  if (f < 0)
  {
    printf("Error in create\n");
    return;
  }
  for (n = 0;n < 1000;n++)
  {
    if (write(f,&c,1) < 0)
    {
      printf("error in write\n");
      close(f);
      return ;
    }
    c++;
    if (c == 127) c = ' ';
  }
  close(f);
}

void * fifo_cons(void * x)
{
  int f,retval;
  char c;
  f = open("cfifo:0",O_CREAT | O_RDWR);
  if (f < 0)
  {
    printf("Error in open\n");
    return NULL;
  }
  //printf("!");
  for (;;)
  {
    retval = read(f,&c,1);
    //printf("C%d",retval);
    if (retval < 0)
    {
      printf("Error in read\n");
      close(f);
      return NULL;
    }
    putchar(c);
  }
  return NULL;
}

extern uint32 timer_ticks;

void * time_test(void * x)
{
  struct timeval tv;
  char  str[80];
  uint16 *vgamem=(uint16*)0xb8000;
  int i;
  signal (1, SIG_IGN);
  //setpriority (PRIO_PROCESS, 0, -1);
  for (;;)
  {
    gettimeofday(&tv,NULL);
    sprintf(str,"%6ld.%06lu",
        tv.tv_sec,tv.tv_usec);
    for (i = 0;i < strlen(str);i++)
      vgamem[i+60] = str[i] + 0x700;
  }
}

void segv(void)
{
	asm("movw $0x0,%ax;movw %ax,%fs;
      movl $0x1,%eax;
      movl $0x2,%ebx;
      movl $0x3,%ecx;
      movl $0x4,%edx;
      movl $0x5,%esi;
      movl $0x6,%edi;
      movl %fs:0,%eax");
}

void ioctl_test(void)
{
  int f;
  f = open("con:",O_RDWR);
  ioctl(f,10,NULL);
  close(f);
}

void bigmem(void)
{
  char * p;
  p = (char *)kmalloc(1440000);
}

uint32 s_proc1(uint32 * p)
{
  printf ("Proc1 %ld\n",p[1]);
}

uint32 s_proc2(uint32 * p)
{
  printf ("Proc2 %ld\n",p[1]);
}

void rpc_svr (void)
{
  t_rpc_server * s;
  s = rpc_create_program ("test",2);
  rpc_register_call (s, "proc1", s_proc1);
  rpc_register_call (s, "proc2", s_proc2);
  rpc_start (s);
}

void test_rpc (void)
{
  int c,i;
  uint32 p[2];
  c = rpc_connect ("test");
  for (i = 0; i < 10;i++)
  {
    p[1] = i;
    rpc_call (c, "proc1", p);
    rpc_call (c, "proc2", p);
  }
}

void s_kill(char * s)
{
  int signum, tid;
  char * p;
  p = strchr(s,' ');
  if (p != NULL)
  {
    signum = -atoi (s);
    tid = atoi (p);
    printf("Sending signal %d to task %d\n",signum, tid);
    kill (tid, signum);
  }
}

void handler (int sig)
{
  printf("Handled signal %d\n",sig);
}


void make_fs (void)
{
  int f, i;
  char block[512];
  printf ("MKNUFS = %d\n", mknufs("ramdisk:0"));
  f = open ("ramdisk:0", O_RDONLY);
  ioctl (f, _IO_BLK_READ (1), block);
  close (f);
  for (i = 0;i < 128; i++)
  {
    if ( (i % 32) == 0) printf ("\n");
    printf ("%02x ", block[i]);
  }
  printf ("\n");
}

void * user_main(void * x)
{
  pthread_t t,tt;
  char  s[80];
  pthread_create(&t,NULL,fifo_cons,NULL);
  if (fork() == 0) bfifo_cons (NULL);
  rpc_svr ();
  //banner();
  signal (1, handler);
  while (1)
  {
    printf("nuxi$ ");
    fgets(s,80,stdin);
    if (strlen(s) > 0) 
    {
      if (!strcmp(s,"clear")) printf("\014");
      else if (!strncmp(s,"cat",3)) cat(s+4);
      else if (!strcmp(s,"fifo")) fifo_prod();
      else if (!strcmp(s,"bfifo")) bfifo_prod();
      else if (!strcmp(s,"segv"))
      {
        if (fork () == 0) segv();
      }
      else if (!strcmp(s,"bigmem")) bigmem();
      else if (!strcmp(s,"mem")) print_mem_space();
      else if (!strcmp(s,"ioctl")) ioctl_test();
      else if (!strcmp(s,"rpc")) test_rpc();
      else if (!strcmp(s,"ps")) cat("sys:tasks");
      else if (!strcmp(s,"bye")) exit(1);
      else if (!strcmp(s,"mkfs")) make_fs ();
      else if (!strcmp(s,"tid"))
        printf ("TID = %ld, PTID = %ld\n",getpid (), getppid ());
      else if (!strncmp(s,"stime",5))
      {
        int newtime;
        newtime = atoi(s+6);
        HZ = newtime;
        i8253_init();
      }
      else if (!strncmp(s,"wait",4))
      {
        int delay;
        delay = atoi(s+5);
        sleep(delay);
      }
      else if (!strncmp(s,"uwait",5))
      {
        long delay;
        delay = atol(s+6);
        usleep(delay);
      }
      else if (!strncmp(s,"kill",4))
      {
        s_kill(s+5);
      }
      else if (!strncmp(s,"ptr",3))
      {
        long delay;
        long *lp;
        lp = &delay;
        printf("PTR = %lx\n",(long)lp);
      }
      else
        printf("You typed: `%s'\n",s);
    }
  }
}

