/*
 * demo.c
 *-
 * Copyright (c) 2001, 2002, 2003 Corrado Santoro (csanto@diit.unict.it)
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Id: demo.c,v 1.1.1.1 2005/05/15 09:50:02 corrado Exp $
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <fcntl.h>
#include <sys/time.h>
#include <unistd.h>
#include <errno.h>
#include <signal.h>
#include <pthread.h>
#include <sys/wait.h>

void * cfifo_consumer (void * x)
{
  int f;
  char c;
  f = open ("cfifo:test_fifo", O_RDONLY);
  if (f < 0)
  {
    fprintf (stderr, "error opening test_fifo: %d\n", errno);
    return NULL;
  }
  while (read (f, &c, 1) == 1)
    putchar (c);
  return NULL;
}

void test_cfifo (void)
{
  pthread_t tid;
  FILE * f;
  char c;
  int  count, status;
  f = fopen ("cfifo:test_fifo", "w+");
  if (f == NULL)
  {
    fprintf (stderr, "error opening test_fifo: %d\n", errno);
    return ;
  }
  pthread_create (&tid, NULL, cfifo_consumer, NULL);
  // producer
  c = ' ';
  for (count = 0; count < 1000; count ++)
  {
    fwrite (&c, 1, 1, f);
    c++;
    if (c == 126) c = ' ';
  }
  // an error means that tid is not yet created (fork not completed),
  // thus wait 1 second (causing a task switch) and retry
  do
  {
    sleep (5);
  }
  while (kill (tid.th_id, SIGINT) < 0);
  wait (&status);
  fclose (f);
}

void test_bfifo (void)
{
  int f;
  char c;
  char buffer[80];
  int  count;
  f = open ("bfifo:test_fifo", O_RDWR);
  if (f < 0)
  {
    fprintf (stderr, "error opening test_fifo: %d\n", errno);
    return ;
  }
  // producer
  c = ' ';
  for (count = 0; count < 10; count ++)
  {
    memset (buffer, c, 40);
    write (f, buffer, 40);
    c++;
    if (c == 126) c = ' ';
  }
  close (f);
}

void * thread_func (void * x)
{
  int i;
  for (i = 0; i < 30;i++)
  {
    printf ("I'm thread %d\n", (int)x);
  }
  return NULL;
}


void test_thread (void)
{
  int i,retval;
  pthread_t th[5];
  for (i = 0;i < 5;i++)
    pthread_create (&th[i],NULL,thread_func, (void *) i);
  for (i = 0;i < 5;i++)
  {
    retval = pthread_join (th[i], NULL);
    printf ("thread end .... %d\n",retval);
  }
}

/*
void test_wait (void)
{
  int tid, status;

  if (fork () == 0)
  {
    sleep (3);
    exit (11);
  }
  else
  {
    printf ("Waiting....\n");
    tid = wait (&status);
    printf ("Task end %d, status %d\n", tid, status);
  }
}

void test_waitpid (void)
{
  int ttid, tid, status;

  if ( (ttid = fork ()) == 0)
  {
    sleep (3);
    exit (3);
  }
  else
  {
    printf ("Waiting....\n");
    tid = waitpid (ttid, &status, 0);
    printf ("Task end %d, status %d\n", tid, status);
  }
}

#include <kernel/wait.h>
#include <kernel/timer.h>

wait_chan test = NULL;

void test_sleep_wait (void)
{
  int retval;
  if (fork () == 0)
  {
    printf ("Going to sleep for 20 seconds...\n");
    retval = thread_timed_sleep (&test, 20000000l);
    printf ("Woken up = %d\n", retval);
    exit (1);
  }
}
*/

void * test_wait_thread (void * x)
{
  sleep (3);
  exit (11);
  return NULL;
}

void test_wait (void)
{
  int status,tid;
  pthread_t ttid;

  pthread_create (&ttid, NULL, test_wait_thread, NULL);
  printf ("Waiting....\n");
  tid = wait (&status);
  printf ("Task end %d, status %d\n", tid, status);
}

void test_waitpid (void)
{
  int status, tid;
  pthread_t ttid;

  pthread_create (&ttid, NULL, test_wait_thread, NULL);
  printf ("Waiting....\n");
  tid = waitpid (ttid.th_id, &status, 0);
  printf ("Task end %d, status %d\n", tid, status);
}

#include <kernel/wait.h>
#include <kernel/timer.h>

wait_chan test = NULL;

void * thread_test_sleep_wait (void * x)
{
  int retval;
  printf ("Going to sleep for 20 seconds...\n");
  retval = thread_timed_sleep (&test, 20000000l);
  printf ("Woken up = %d\n", retval);
  exit (1);
  return NULL;
}

void test_sleep_wait (void)
{
  pthread_t tid;
  pthread_create (&tid, NULL, thread_test_sleep_wait, NULL);
}

void test_wakeup (void)
{
  thread_wakeup_all (&test);
}

int demo_main (int argc, char * argv[])
{
  char * p = NULL;
  if (argc < 2)
  {
    fprintf (stderr, "usage: %s feature-to-test\n", argv[0]);
    return 1;
  }
  if (!strcmp (argv [1],"cfifo"))
    test_cfifo ();
  else if (!strcmp (argv [1],"bfifo"))
    test_bfifo ();
  else if (!strcmp (argv [1],"alloc"))
    p = malloc (1023);
  else if (!strcmp (argv [1],"free"))
    free (p);
  else if (!strcmp (argv [1],"thread"))
    test_thread ();
  else if (!strcmp (argv [1],"wait"))
    test_wait ();
  else if (!strcmp (argv [1],"waitpid"))
    test_waitpid ();
  else if (!strcmp (argv [1],"sleep"))
    sleep (5);
  else if (!strcmp (argv [1],"gotosleep"))
    test_sleep_wait ();
  else if (!strcmp (argv [1],"wakeup"))
    test_wakeup ();
  else
    fprintf (stderr, "%s: invalid feature\n", argv[1]);
  return 0;
}

